Package: efdm
Title: Simulate Forest Resources with the European Forestry Dynamics
        Model
Version: 0.2.3
Authors@R: c(
    person("Mikko", "Kuronen", email="mikko.kuronen@luke.fi", role=c("aut", "cre"),
           comment = c(ORCID = "0000-0002-8089-7895")),
    person("Minna", "Räty", email="minna.raty@luke.fi", role="aut",
           comment = c(ORCID = "0000-0001-9898-8712")))
Description: An implementation of European Forestry Dynamics Model (EFDM) and
    an estimation algorithm for the transition probabilities.
    The EFDM is a large-scale forest model that simulates the development of
    the forest and estimates volume of wood harvested for any given forested
    area. This estimate can be broken down by, for example, species, site
    quality, management regime and ownership category.
    See Packalen et al. (2015) <doi:10.2788/153990>.
URL: https://github.com/mikkoku/efdm
BugReports: https://github.com/mikkoku/efdm/issues
License: GPL-2
Encoding: UTF-8
LazyData: true
Imports: stats, utils, data.table
Suggests: dplyr, ggplot2, gridExtra, knitr, rmarkdown, sf, testthat
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Depends: R (>= 2.10)
NeedsCompilation: no
Packaged: 2026-04-24 05:09:23 UTC; 03052156
Author: Mikko Kuronen [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8089-7895>),
  Minna Räty [aut] (ORCID: <https://orcid.org/0000-0001-9898-8712>)
Maintainer: Mikko Kuronen <mikko.kuronen@luke.fi>
Repository: CRAN
Date/Publication: 2026-04-27 05:30:03 UTC
Built: R 4.5.2; ; 2026-05-01 17:10:43 UTC; unix

Package: efdm
Title: Simulate Forest Resources with the European Forestry Dynamics
        Model
Version: 0.2.3
Authors@R: c(
    person("Mikko", "Kuronen", email="mikko.kuronen@luke.fi", role=c("aut", "cre"),
           comment = c(ORCID = "0000-0002-8089-7895")),
    person("Minna", "Räty", email="minna.raty@luke.fi", role="aut",
           comment = c(ORCID = "0000-0001-9898-8712")))
Description: An implementation of European Forestry Dynamics Model (EFDM) and
    an estimation algorithm for the transition probabilities.
    The EFDM is a large-scale forest model that simulates the development of
    the forest and estimates volume of wood harvested for any given forested
    area. This estimate can be broken down by, for example, species, site
    quality, management regime and ownership category.
    See Packalen et al. (2015) <doi:10.2788/153990>.
URL: https://github.com/mikkoku/efdm
BugReports: https://github.com/mikkoku/efdm/issues
License: GPL-2
Encoding: UTF-8
LazyData: true
Imports: stats, utils, data.table
Suggests: dplyr, ggplot2, gridExtra, knitr, rmarkdown, sf, testthat
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Depends: R (>= 2.10)
NeedsCompilation: no
Packaged: 2026-04-24 05:09:23 UTC; 03052156
Author: Mikko Kuronen [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8089-7895>),
  Minna Räty [aut] (ORCID: <https://orcid.org/0000-0001-9898-8712>)
Maintainer: Mikko Kuronen <mikko.kuronen@luke.fi>
Repository: CRAN
Date/Publication: 2026-04-27 05:30:03 UTC

# Generated by roxygen2: do not edit by hand

export("transprobs<-")
export(define_activity)
export(estimatetransprobs)
export(prior_ff)
export(prior_grow)
export(runEFDM)
export(transprobs)
importFrom(data.table,.SD)
importFrom(data.table,as.data.table)
importFrom(stats,aggregate)
importFrom(utils,head)

# efdm
An implementation of European Forestry Dynamics Model (EFDM) and
    an estimation algorithm for the transition probabilities.
    The EFDM is a large-scale forest model that simulates the development of
    the forest and estimates volume of wood harvested for any given forested
    area. This estimate can be broken down by, for example, species, site
    quality, management regime and ownership category.

The European Forestry Dynamics Model: Concept, design and results of first case studies, Packalen et al. DOI:[10.2788/153990](http://dx.doi.org/10.2788/153990)

# Installation

The package is available from CRAN [https://cran.r-project.org/package=efdm].
```R
install.packages("efdm")
vignette("example")
```

# Vignette

Vignette is also available outside R.

[Scenario example](https://mikkoku.github.io/efdm/vignettes/example.html)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
options(max.print=50)
data.table::setDTthreads(1)

## -----------------------------------------------------------------------------
library(efdm)

## ----setup, warning = FALSE, message = FALSE----------------------------------
library(dplyr)
library(ggplot2)
library(sf)

## -----------------------------------------------------------------------------
data(example)

## -----------------------------------------------------------------------------
actprob <- example$actprob
actprob

## -----------------------------------------------------------------------------
transprobs_ff <- expand.grid(vol0=1:15, age0=1:35, vol1=1, age1=1, prob=1)
transprobs_ff

## ----ff-----------------------------------------------------------------------
ff <- define_activity("ff", c("vol", "age"), transprobs_ff)

## ----pairdata-----------------------------------------------------------------
example$noman_pairs

## ----statespace---------------------------------------------------------------
statespace <- actprob %>% select(c(region, soil, sp, vol, age))
statespace

## -----------------------------------------------------------------------------
transprobs_noman <- estimatetransprobs(c("vol", "age"), # the dynamic variables of the activity
                     example$noman_pairs,  # pair data for no management
                     statespace=statespace,
                     factors=c("soil"), # Information for different soil types
                                        # is used with smaller weight
                     by=c("region","sp"), # Separate estimation for each
                                          # region and species
                     prior=prior_grow("age"))

## ----noman--------------------------------------------------------------------
noman <- define_activity("noman", c("vol", "age"), transprobs_noman)

## -----------------------------------------------------------------------------
transprobs_noman %>%
  arrange(vol0, age0, soil, region, sp, vol1, age1)

## ----thin---------------------------------------------------------------------
transprobs_thin <- estimatetransprobs(c("vol", "age"),
                     example$thin_pairs, # pair data for thinning
                     statespace=statespace,
                     factors=c("soil"),
                     by=c("region","sp"),
                     prior=prior_grow("age"))
thin <- define_activity("thin", c("vol", "age"), transprobs_thin)

## -----------------------------------------------------------------------------
state0 <- example$initial_state
state0

## -----------------------------------------------------------------------------
activities <- list(noman, thin, ff)
states1 <- runEFDM(state0, actprob, activities, 20)

## -----------------------------------------------------------------------------
states1 %>%
  arrange(soil, region, sp, vol, age, time, activity)

## -----------------------------------------------------------------------------
example$vol_coef %>%
  arrange(vol, sp, species)

## -----------------------------------------------------------------------------
example$drain_coef %>%
  arrange(-vol, sp, assort, activity)

## -----------------------------------------------------------------------------
example$income_coef

## -----------------------------------------------------------------------------
states1 <- states1 %>% mutate(time = factor(2016 + time*5))

## -----------------------------------------------------------------------------
volume <- merge(states1, example$vol_coef) %>%
  mutate (volume=area*volume) %>% #area is multiplied with m3/ha volume the get total volume in m3
  filter(species != "all")
ggplot(volume) +
  scale_fill_viridis_d(end = 0.9) +
  geom_bar(aes(x=time, weight=volume/1000000000, fill=species)) +
  labs(y=NULL,title=expression(paste("Growing stock, bil.",m^3)), x="Year", fill="") +
  theme(legend.position = "bottom", axis.text.x = element_text(angle = -90))

## -----------------------------------------------------------------------------
states1$ageclass <- cut(states1$age, breaks=c(0,10,20,30,35), include.lowest = TRUE, 
                        #labels=c("0-50","51-100","101-150","150+"))
                        labels=c("-50","-100","-150","150+"))
states1$region <- factor(states1$region, labels = c("South", "Middle", "North"))
ggplot(subset(states1, time %in% c(2016,2066,2116))) +
  geom_bar(aes(x=ageclass, weight=area/1000000, fill=region)) +  
  scale_fill_viridis_d(end = 0.9) +
  facet_grid(cols=vars(time)) +
  labs(y=NULL, title="Area, mill.ha", x="Ageclass", fill=NULL) +
  theme(legend.position = "bottom", axis.text.x = element_text(angle = -90))

## -----------------------------------------------------------------------------
euro <- merge(example$drain_coef, example$income_coef) %>% mutate(euro = euro*drain)
removal <- merge(states1, euro) %>% mutate(income=euro*area)

ggplot(subset(removal,!time %in% c(2116))) + 
  geom_bar(aes(x=time, weight=income/5000000000, fill=assort)) +
  scale_fill_viridis_d(end = 0.9) +
  labs(y=NULL,title = "Income, bil.€/year", x="5-year intervals", fill=NULL ) +
  theme(legend.position = "bottom", axis.text.x = element_text(angle = -90))

## -----------------------------------------------------------------------------
transprobs_ff_age_species <- statespace %>%
  select(vol0=vol, age0=age, sp0=sp, region) %>%
  unique %>%
  group_by(region, sp0, vol0, age0) %>%
  reframe(data.frame(vol1=1, age1=1, sp1=c('other','spruce'),
                       prob=case_when(
                         sp0=='other' && region=='South' ~ c(1, 0),
                         sp0=='other' && region=='Middle' ~ c(1, 0),
                         sp0=='other' && region=='North' ~ c(0.8, 0.2), #in other dominated forest
                                        # in northern boreal vegetation zone 20% of final felled area
                                        # change to spruce dominated forest
                         sp0=='spruce' && region=='South' ~ c(0.3, 0.7),
                         sp0=='spruce' && region=='Middle' ~ c(0.2, 0.8),
                         sp0=='spruce' && region=='North' ~ c(0, 1))))
ff_age_species <- define_activity("ff", c("vol", "age", "sp"), transprobs_ff_age_species)

## -----------------------------------------------------------------------------
activities2 <- list(noman, thin, ff_age_species)
states2 <- runEFDM(state0, actprob, activities2, 20)

## -----------------------------------------------------------------------------
# Compute proportion of spruces in each time and region
prop_spruces <- states2 %>%
  group_by(region, time) %>%
  summarise(.groups="drop", proportion = 100*sum((sp=="spruce")*area)/sum(area)) %>%
  filter(time %in% c(0, 10, 20)) %>%
  mutate(time = 2016 + 5*time)

# Add bio-geographical regions (MetsaKasvVyoh provided in efdm package) to the data
prop_spruces <- merge(MetsaKasvVyoh, prop_spruces)
prop_spruces %>% ggplot() +
  geom_sf(aes(fill=proportion)) +
  facet_wrap(vars(time)) +
  scale_fill_gradient(low = "white", high = "forestgreen") +
  labs(fill="Proportion" ,title="Proportion of spruce dominated forests, %") +
  guides(fill = guide_colourbar(frame.colour = "grey20",ticks.colour = "grey20")) +
  theme(legend.position = "bottom", axis.text.x = element_text(angle = -90))

## -----------------------------------------------------------------------------
statespace3 <- statespace
statespace3$landuse <- "forest"
agriculture <- unique(statespace3 %>% mutate(sp=0, vol=0, age=0, landuse="agriculture"))
other <- unique(statespace3 %>% mutate(soil=0, sp=0, vol=0, age=0, landuse="other"))
statespace3 <- rbind(statespace3, agriculture, other)

## -----------------------------------------------------------------------------
transprobs_defor_to_agri <- statespace3 %>%
  filter(landuse=="forest") %>%
  select(vol0=vol, age0=age, sp0=sp,landuse0=landuse) %>%
  unique %>%
  mutate(vol1=0, age1=0, sp1=0, landuse1="agriculture", prob=1)
defor_to_agri <- define_activity("defor_to_agri",
                                 c("vol", "age", "sp", "landuse"),
                                 transprobs_defor_to_agri)

## -----------------------------------------------------------------------------

transprobs_defor_to_other <- statespace3 %>%
  filter(landuse=="forest") %>%
  select(soil0=soil, vol0=vol, age0=age, sp0=sp,landuse0=landuse) %>%
  unique %>%
  mutate(soil1=0, vol1=0, age1=0, sp1=0, landuse1="other", prob=1)
defor_to_other <- define_activity("defor_to_other",
                                  c("soil", "vol", "age", "sp", "landuse"),
                                  transprobs_defor_to_other)

## -----------------------------------------------------------------------------
transprobs_aff <- statespace3 %>%
  filter(landuse=="agriculture") %>%
  select(soil, vol0=vol, age0=age, region, sp0=sp, landuse0=landuse) %>%
  mutate(vol1=1, age1=1, sp1="other", landuse1="forest", prob=1)
transprobs_aff <- rbind(transprobs_aff %>% mutate(sp1="spruce", prob=0.5),
                        transprobs_aff %>% mutate(sp1="other", prob=0.5))
affor <- define_activity("affor",
                         c("vol", "age", "sp", "landuse"),
                         transprobs_aff)

## -----------------------------------------------------------------------------
donothing <- define_activity("donothing", character())
activities3 <- list(noman, thin, ff, defor_to_other, defor_to_agri, affor, donothing)

## -----------------------------------------------------------------------------
state03 <- state0
state03$landuse <- "forest"
state03 <- rbind(state03,  agriculture %>% mutate(area=rep(c(1552000/2,997000/2,76000/2),each=2)),
                 other %>% mutate(area=c(721000,507000,112000)))

## -----------------------------------------------------------------------------
actprob3 <- actprob
actprob3$landuse <- "forest"
# Define probabilities for new activities
actprob3$defor_to_other <- 0.0002
actprob3$defor_to_agri <- 0.00025
actprob3$affor <- 0
actprob3$donothing <- 0
# Normalise probabilities after adding new activities
actnames <- c("noman", "thin", "ff", "defor_to_other", "defor_to_agri", "affor", "donothing")
actprob3[actnames] <- actprob3[actnames]/rowSums(actprob3[actnames])
# Define activity probabilities also for agriculture and other land uses
actprob3 <- rbind(actprob3, 
        agriculture %>% mutate(thin=0, ff=0, noman=0, defor_to_other=0, defor_to_agri=0, affor=0.0005, donothing=0.9995),
        other %>% mutate(thin=0, ff=0, noman=0, defor_to_other=0, defor_to_agri=0, affor=0, donothing=1))

## -----------------------------------------------------------------------------
states3 <- runEFDM(state03, actprob3, activities3, 20)

## -----------------------------------------------------------------------------
#pie charts of land use areas
LUareas <- states3 %>%
  group_by(time, landuse) %>%
  summarise(.groups="drop", area = sum(area)) %>%
  ungroup()
LUareas <- LUareas %>%
  mutate(time = factor(time, labels = seq(2016,2116,5)))
totalarea <- sum(state03$area)
LUareas %>% filter(time %in% c("2016","2066","2116")) %>%
  mutate(area = area/totalarea) %>%
  ggplot() +
  geom_bar(aes(x="",y=area, fill=landuse), stat="identity", width=1, color = "white") +
  scale_fill_viridis_d(end = 0.9) +
  coord_polar("y", start=0) +
  facet_wrap(vars(time)) +
  labs(y=NULL,fill="Land use", x=NULL)+
  theme(axis.text.x = element_blank(),
        axis.ticks = element_blank()) +
  geom_text(aes(x="",y = rep(c(0.03, 0.33, 0.95), 3), label = round(area*100,1)),
            nudge_x=0.25, size=2.25,
            color=rep(c("grey20","white","white"), 3), fontface="bold") +
  theme(legend.position = "bottom")

#' Estimate Transition Probabilities from Pairdata
#'
#' Estimate Transition Probabilities from Pairdata
#'
#' Transition probabilities 'move' the forest areas allocated in the cells of state matrix
#' from the initial states in the beginning of a EFDM run step to the end position. This end
#' position will be the initial state of the next EFDM step. Length of a step (=time)
#' in the EFDM run is typically determined by the pairdata. It is the time difference of
#' tree observations. Note that the pairdata can be also constructed from single observation,
#' if the other (pair) observation is estimated or modelled.
#'
#' Each activity needs to have a transition probability. If no pairdata is available,
#' transition probability matrices can be based entirely on a prior defined with
#' expert knowledge.
#'
#' The estimation uses an iterative Bayesian algorithm that is explained in
#' \url{https://github.com/ec-jrc/efdm/blob/master/documents/EFDMinstructions/Seija_Mathematics_behind_EFDM.pdf}.
#' The transition probability estimate is the proportion of observed transitions
#' divided by the number of all transitions from the same starting state.
#' \code{prior} gives the number of prior transitions. For each \code{factor}
#' variable the transitions are counted in classes of all factors before the
#' current factor. The "most important" observations (having all classes right)
#' is counted \code{length(factors)} times, the second most important observations
#' are counted \code{length(factors)-1} times and so on.
#'
#' If pairdata is NULL prior is used by itself.
#'
#' Observations should have 'factor' and 'by' variables and statepairs with 0 and 1
#' suffixes to indicate before and after observations.
#'
#' The estimation algorithm uses information across 'factor' variables, but not
#' across 'by' variables.
#'
#' \code{prior} can either \code{character} or \code{function}.
#' \itemize{
#' \item "nochange" implies that there is one observation where state doesn't change
#' \item "uninformative" when no observations are given all states are as likely
#' \item \code{function(A, dynvar1, dynvar0)} where A is an array of zeros with
#'   dimnames(A) <- c(dynvar1, dynvar0). The function should fill A with the number of
#'   prior transitions and return it.
#' }
#'
#' The statespace is used to fill in the transitions where there are no observations.
#' In special cases the statespace may change as a result of the activity.
#' For example changing the tree species might lead to change in the volume classification used.
#'
#' This function assumes that the dynamic variables are coded as integers starting with 1.
#' Other variables are not restricted.
#'
#' @param dynamicvariables The names of the dynamic variables, \code{character} vector
#' @param pairdata \code{data.frame} Observed transitions
#' @param statespace \code{data.frame} or a \code{list} of two \code{data.frame}s if the state space changes as a result of the activity
#' @param factors \code{character} Variables used by the activity
#' @param by \code{character} Variables that split the state space
#' @param prior function or character
#' @return Transition probabilities as a \code{data.frame}
#' @examples
#'
#' # Estimation can use observed transitions with different levels of factors.
#' statespace <- expand.grid(a=1:2, b=1:2, vol=1:5)
#' pairdata <- data.frame(a=c(1,1,2,2), b=c(1,2,1,2), vol0=c(1,1,1,1), vol1=c(2,3,4,5))
#' state0 <- statespace
#' actprob <- statespace
#' actprob$test <- 1
#' state0$area <- 0
#' state0$area[1] <- 1
#'
#' # With by=c("a", "b") there are two observations: one from prior and the other
#' # from the exact combination of class levels.
#' probs <- estimatetransprobs("vol", pairdata, statespace, by=c("a", "b"), prior="nochange")
#' act1 <- define_activity("test", c("vol"), probs)
#' runEFDM(state0, actprob, list(act1), 1)
#'
#' probs <- estimatetransprobs("vol", pairdata, statespace, factors="a", by="b", prior="nochange")
#' act2 <- define_activity("test", c("vol"), probs)
#' runEFDM(state0, actprob, list(act2), 1)
#'
#' probs <- estimatetransprobs("vol", pairdata, statespace, factors="b", by="a", prior="nochange")
#' act3 <- define_activity("test", c("vol"), probs)
#' runEFDM(state0, actprob, list(act3), 1)
#'
#' # The order of variables in factors argument specifies the order of importance.
#' # Observation that differ in the first variable are counted more times.
#' probs <- estimatetransprobs("vol", pairdata, statespace, factors=c("a", "b"), prior="nochange")
#' act4 <- define_activity("test", c("vol"), probs)
#' runEFDM(state0, actprob, list(act4), 1)
#'
#' probs <- estimatetransprobs("vol", pairdata, statespace, factors=c("b", "a"), prior="nochange")
#' act5 <- define_activity("test", c("vol"), probs)
#' runEFDM(state0, actprob, list(act5), 1)
#'
#'
#' @export
estimatetransprobs <- function(dynamicvariables, pairdata,
                               statespace,
                               factors=character(), by=character(),
                               prior="nochange") {
  act <- define_activity("temp", dynamicvariables)
  if (is.data.frame(statespace)) {
    statespace0 <- statespace1 <- statespace
  } else {
    statespace0 <- statespace[[1]]
    statespace1 <- statespace[[2]]
  }
  act <- build_complex_statespace(act, statespace0, statespace1, factors, by)
  dynvar0 <- act$dynamicvariables0
  dynvar1 <- act$dynamicvariables1
  statespace <- act$statespace

  if(!is.null(pairdata)) {
    requirednames <- c(dynvar0, dynvar1, factors, by)
    missingnames <- setdiff(requirednames, names(pairdata))
    if(length(missingnames) > 0) stop(paste0("Variable(s) '", list(missingnames), "' not present in pairdata."))

    # Check pairdata against statespace
    a <- act$statespace0
    a$check_missing_combinations <- 1
    a <- merge(pairdata, a, by.x = c(factors, by, act$dynamicvariables0),
          by.y = c(factors, by, gsub("0$", "", act$dynamicvariables0)), all.x=TRUE)
    if(anyNA(a$check_missing_combinations)) stop("Pairdata has transitions starting from states that are not in statespace.")

    a <- act$statespace1
    a$check_missing_combinations <- 1
    a <- merge(pairdata, a, by.x = c(factors, by, act$dynamicvariables1),
               by.y = c(factors, by, gsub("1$", "", act$dynamicvariables1)), all.x=TRUE)
    if(anyNA(a$check_missing_combinations)) {
      print("Pairdata has transitions resulting in states that are not in statespace.")
      print(utils::head(a[is.na(a$check_missing_combinations),,drop=FALSE]))
      stop("Pairdata has transitions resulting in states that are not in statespace.")
    }
  }

  pairdata <- pairdata[c(dynvar1, dynvar0, factors, by)]
  processed_rows <- 0

  A <- do.call(rbind, lapply(statespace, function(statespacepart) {
    bydata <- NULL
    priorpart <- prior
    if(!is.null(pairdata)) {
      #mergeby <- c(factors, by) # Why not just 'by' here?
      mergeby <- by
      if(length(mergeby)) {
        select <- unique(statespacepart$statespace0[mergeby])
        bydata <- as.data.frame(merge(as.data.table(pairdata), as.data.table(select), all=FALSE, by=mergeby))
        if(is.data.frame(prior)) {
          priorpart <- merge(prior, select, all=FALSE, by=mergeby)[c(act$dynamicvariables0, act$dynamicvariables1)]
        }
      } else {
        bydata <- pairdata
      }
      processed_rows <<- processed_rows + nrow(bydata)
    }
    estimatetransprobs1(bydata, statespacepart, priorpart, dynvar0, dynvar1, factors)
  }))
  if(!is.null(pairdata)) {
    if(processed_rows > nrow(pairdata)) stop("Internal error. Processed pairdata multiple times.")
    if(processed_rows < nrow(pairdata)) warning("Not all pairdata was processed. Was this intentional?")
  }
  A$N <- NULL
  A$nobs <- NULL
  A
}

build_complex_statespace <- function(act, statespace0, statespace1, factors=character(), by=character()) {
  extranames <- setdiff(names(statespace0), c(factors, by, gsub("0$", "", act$dynamicvariables0)))
  if(length(extranames)) stop(paste0("Variables '", list(extranames), "' in statespace not used by activity."))
  act$factors <- factors
  act$by <- by
  act$statespace0 <- statespace0
  act$statespace1 <- statespace1
  act$statespace <- build_statespace_by(statespace0, statespace1, act$dynamicvariables0, act$dynamicvariables1, factors, by)
  act
}
build_statespace_by <- function(statespace0, statespace1, dynamicvariables0, dynamicvariables1, factors, by) {
  if(!all(by %in% names(statespace1))) stop("All by variables should be in statespace1")
  if(!all(gsub("0$", "", dynamicvariables0) %in% names(statespace0))) stop("All dynamic variables should be in statespace0")
  if(!all(gsub("1$", "", dynamicvariables1) %in% names(statespace1))) stop("All dynamic variables should be in statespace1")
  if(!all(by %in% names(statespace0))) stop("All by variables should be in statespace0")

  if(length(by)==0) return(list(list(statespace0=statespace0, statespace1=statespace1)))
  byrows <- unique(rbind(statespace0[by], statespace1[by]))
  return(lapply(1:nrow(byrows), function(i) {
    #TODO: Change merge to a simple subset
    # merge has an interesting view on "sort"ing
    bys0 <- as.data.frame(merge(as.data.table(byrows[i, , drop=FALSE]), as.data.table(statespace0), by=by, all=FALSE))
    bys1 <- as.data.frame(merge(as.data.table(byrows[i, , drop=FALSE]), as.data.table(statespace1), by=by, all=FALSE))
    list(statespace0=bys0, statespace1=bys1)
  }))
}


# Matrix form of prior should only work with numeric states
# => no need to worry about factors
apply_prior <- function(statespace, prior, dynvar0, dynvar1) {
  v0 <- sapply(dynvar0, function(v) {list(sort(unique(statespace$statespace0[[gsub("0$", "", v)]])))})
  v1 <- sapply(dynvar1, function(v) {list(sort(unique(statespace$statespace1[[gsub("1$", "", v)]])))})

  if(is.function(prior)) {
    A <- array(0, dim = sapply(c(v1, v0), length))
    dimnames(A) <- c(v1, v0)
    prior <- prior(A, v1, v0)
    if(!identical(dim(A), dim(prior))) stop("Prior function didn't return array of right size.")
    if(!identical(dimnames(A), dimnames(prior))) stop("Prior function didn't return array with right dimnames.")
  } else if(prior == "nochange") {
    if(!identical(unname(v0), unname(v1))) stop("nochange prior is only available if statespaces are identical")
    prior <- diag(prod(sapply(v0, length)))
    dim(prior) <- sapply(c(v1, v0), length)
    dimnames(prior) <- c(v1, v0)
  } else if(prior == "uninformative") {
    prior <- array(1e-15, dim = sapply(c(v1, v0), length))
    dimnames(prior) <- c(v1, v0)
  } else {
    stop("Only nochange and uninformative prior is implemented")
  }
  check <- prior
  dim(check) <- c(prod(sapply(v1, length)), prod(sapply(v0, length)))
  if(any(colSums(check)==0)) stop("Prior gave zero transition probabilities when starting from some states. Please check your prior.")

  out <- expand.grid(c(v1, v0), stringsAsFactors = FALSE)
  out$nobs <- c(prior)
  out <- out[out$nobs != 0,]
  out <- merge(out, statespace$statespace0, by.x=dynvar0, by.y=gsub("0$", "", dynvar0), sort=FALSE)
  out
}

estimatetransprobs1 <- function(pairdata, statespace, prior, dynvar0, dynvar1, factors) {

  nobs <- if(is.data.frame(prior)) {
    prior
  } else {
    apply_prior(statespace, prior, dynvar0, dynvar1)
  }

  if(!is.null(pairdata) && nrow(pairdata) > 0) {
    # Reorder columns for iterativebayesian estimation
    usednames <- c(dynvar1, dynvar0, factors)

    pairdatafact <- pairdata[usednames]
    # Factors need all levels counted to borrow information across levels
    # dynamic variables only those with observations.
    # factors need to be converted to factor, because different 'by' classes can have different factor levels.
    for(n in factors) {
      pairdatafact[[n]] <- factor(pairdata[[n]], levels=sort(unique(statespace$statespace0[[n]])))
    }
    nobs2 <- iterativebayesestimation(pairdatafact, 0, length(dynvar0), length(dynvar1))
    # do as.data.frame(nobs2), but without converting numberic variables to factors
    vars <- c(sapply(c(dynvar1, dynvar0), function(x) list(sort(unique(pairdata[[x]])))),
      sapply(factors, function(x) list(sort(unique(statespace$statespace0[[x]])))))
    grid <- expand.grid(vars)#, stringsAsFactors = FALSE)
    grid$nobs <- c(nobs2)
    grid <- grid[grid$nobs != 0,]
    # Add by variables
    for(n in setdiff(names(pairdata), usednames)) {
      grid[[n]] <- pairdata[[n]][1]
    }
    if(length(factors)) {
      grid <- merge(grid, unique(statespace$statespace0[factors]), by=factors, all=FALSE)
    }
    nobs <- merge(nobs, grid, by=names(pairdata), suffixes=c("", ".y_check_839"), all=TRUE)
    na20 <- function(x) {x[is.na(x)] <- 0; x}
    nobs$nobs <- na20(nobs$nobs) + na20(nobs$nobs.y_check_839)
    nobs$nobs.y_check_839 <- NULL
    if(length(grep("\\.y_check_839$", names(nobs)))) stop(paste0("Internal error. Extra variables after transition matrix estimation."))
  }

  N <- aggregate(list(N=nobs$nobs), nobs[c(dynvar0, factors)], sum)
  p <- merge(nobs, N, by=c(dynvar0, factors))
  p$prob <- p$nobs / p$N
  sump <- nrow(statespace$statespace0)
  if(!isTRUE(all.equal(sum(p$prob),sump))) {
    stop(paste0("Sum of estimated transition probabilities is ", sum(p$prob), " but should be ", sump))
  }
  return(p)
}

iterativebayesestimation <- function(pairdata, prior, ls0, ls1) {
  n <- prior
  # Count number of observations with different factors
  # The observations with more specific factor combinations are counted more
  # times than those with less specific combinations.
  for (i in (ls0+ls1):length(pairdata))
  {
    tmp <- table(pairdata[1:i])
    # Since the new dimension is the last dimension it is possible to use
    # recycling to add the previous array to each level of the new dimension
    tmp[] <- c(tmp) + c(n)
    n <- tmp
  }
  n
}

data.table::setDTthreads(1)
library(testthat)
library(efdm)
statespace <- expand.grid(ds=c("sp", "pi"), region=c("n", "s"), vol=1:3, stringsAsFactors=FALSE)
pairdata <- data.frame(ds=c("sp", "pi"), vol0=c(1,1), vol1=c(2,3), region=c("n", "s"))
actprob <- state0 <- statespace
actprob$test <- 1
state0$area <- c(1,1,1,1, 0,0,0,0, 0,0,0,0)
orderresult <- function(res) {
  res <- res[do.call(order, res[c("time", "region", "ds", "vol")]),c("time", "region", "ds", "vol", "activity", "area")]
  row.names(res) <- NULL
  res
}

act1 <- define_activity("test", c("vol"))
transprobs(act1) <- estimatetransprobs("vol", NULL, subset(statespace, ds=="sp"&region=="n"), factors=c("ds", "region"), prior="nochange")
act2 <- define_activity("test", c("vol"))
transprobs(act2) <- estimatetransprobs("vol", NULL, subset(statespace, !(ds=="sp"&region=="n")), factors=c("ds", "region"), prior="nochange")
orderresult(runEFDM(state0, actprob, list(act1, act2), 1))

act1 <- define_activity("test", c("vol"))
transprobs(act1) <- estimatetransprobs("vol", subset(pairdata, ds=="sp"&region=="n"), subset(statespace, ds=="sp"&region=="n"), factors=c("ds", "region"), prior="nochange")
act2 <- define_activity("test", c("vol"))
transprobs(act2) <- estimatetransprobs("vol", subset(pairdata, !(ds=="sp"&region=="n")), subset(statespace, !(ds=="sp"&region=="n")), factors=c("ds", "region"), prior="nochange")
orderresult(runEFDM(state0, actprob, list(act1, act2), 10))


statespace <- expand.grid(ds=c("sp", "pi"), region=c("n", "s"), vol=1:3, stringsAsFactors=FALSE)
act <- define_activity("test", c("vol"))
statespace1 <- subset(statespace, !(ds=="sp"&region=="n"))
act <- efdm:::build_complex_statespace(act, statespace1, statespace1, factors=c("ds", "region"))
stopifnot(nrow(act$statespace[[1]])==3)

statespace <- expand.grid(a=1:2, b=c("f", "g"), c=c(4,9), vol=1:3, stringsAsFactors=FALSE)
ii <- sample(nrow(statespace), sample(nrow(statespace), 1))
act <- define_activity("test", c("vol"))
act <- efdm:::build_complex_statespace(act, statespace[ii,], statespace[ii,], factors=c("a", "b"), by="c")
expect_identical(sum(sapply(act$statespace, function(x) nrow(x$statespace0))), length(ii))

data.table::setDTthreads(1)
library(efdm)
statespace <- expand.grid(a=1:2, b=1:2, vol=1:5)
pairdata <- data.frame(a=c(1,1,2,2), b=c(1,2,1,2), vol0=c(1,1,1,1), vol1=c(2,3,4,5))
state0 <- statespace
actprob <- statespace
actprob$test <- 1
state0$area <- 0
state0$area[1] <- 1

act1 <- define_activity("test", c("vol"))
transprobs(act1) <- estimatetransprobs("vol", pairdata, statespace, by=c("a", "b"), prior="nochange")
transprobs(act1)
runEFDM(state0, actprob, list(act1), 1)
act2 <- define_activity("test", c("vol"))
transprobs(act2) <- estimatetransprobs("vol", pairdata, statespace, factors="a", by="b", prior="nochange")
transprobs(act2)
runEFDM(state0, actprob, list(act2), 1)
act3 <- define_activity("test", c("vol"))
transprobs(act3) <- estimatetransprobs("vol", pairdata, statespace, factors="b", by="a", prior="nochange")
transprobs(act3) |> dplyr::arrange(a, vol0, b, vol1)
runEFDM(state0, actprob, list(act3), 1)
act4 <- define_activity("test", c("vol"))
transprobs(act4) <- estimatetransprobs("vol", pairdata, statespace, factors=c("a", "b"), prior="nochange")
transprobs(act4) |> dplyr::arrange(vol0, a, b, vol1)
runEFDM(state0, actprob, list(act4), 1)
act5 <- define_activity("test", c("vol"))
transprobs(act5) <- estimatetransprobs("vol", pairdata, statespace, factors=c("b", "a"), prior="nochange")
transprobs(act5) |> dplyr::arrange(vol0, b, a, vol1)
runEFDM(state0, actprob, list(act5), 1)

act6 <- define_activity("test", c("vol"))
transprobs(act6) <- estimatetransprobs("vol", pairdata[c("vol0", "vol1")], data.frame(vol=1:5), prior="nochange")
transprobs(act6)
runEFDM(state0, actprob, list(act6), 1)

try(estimatetransprobs("vol", pairdata[c("vol0", "vol1")], data.frame(vol=1:4), prior="nochange"))

try(estimatetransprobs("Vol", pairdata[c("vol0", "vol1")], data.frame(vol=1:4), prior="nochange"))

try(estimatetransprobs("Vol", pairdata[c("vol0", "vol1")], data.frame(Vol=1:4), prior="nochange"))


## TODO: add the above tests for these too

statespace <- expand.grid(a=c("pine", "spruce"), b=1:2, vol=1:5)
pairdata <- data.frame(a=c("pine", "pine", "spruce", "spruce"), b=c(1,2,1,2), vol0=c(1,1,1,1), vol1=c(2,3,4,5))
state0 <- statespace
actprob <- statespace
actprob$test <- 1
state0$area <- 0
state0$area[1] <- 1


statespace <- expand.grid(a=as.factor(c("pine", "spruce")), b=1:2, vol=1:5)
pairdata <- data.frame(a=as.factor(c("pine", "pine", "spruce", "spruce")), b=c(1,2,1,2), vol0=c(1,1,1,1), vol1=c(2,3,4,5))
state0 <- statespace
actprob <- statespace
actprob$test <- 1
state0$area <- 0
state0$area[1] <- 1

#' Define an activity
#'
#' Define an activity
#'
#' The set of activities in EFDM defines all possible alternatives for a forest
#' stratum to develop during a scenario run step. Therefore activities are not only
#' limited to forest treatments and management actions such as thinnings and final fellings
#' but should also include 'no management' i.e. growth, if applicable. An activity may
#' also be something else affecting the development, for example, a forest hazard:
#' snow, wind, drought, pest damage etc.
#'
#' \code{name} is used as the name of the activity in the result of \code{\link{runEFDM}}
#' and it is used to identify the activity probability.
#' If multiple activities should be reported under a common name, \code{actprobname} can
#' be used to specify the activity name in activity probability data.
#'
#' \code{dynamicvariables} are those (state space) variables which change as
#' a result of the activity
#' Typically an activity affects on the age, volume or stem count of the
#' forest, but an activity may also, for example, change land-use and then
#' a variable related to land-use categories is essential.
#'
#' The \code{transprobs} should be a \code{data.frame} with columns:
#' \itemize{
#' \item pairs of dynamic variables. For example, if the dynamic variables are
#'   \code{vol} and \code{age}, then transprobs should have columns
#'   \code{age0}, \code{age1}, \code{vol0}, \code{vol1}.
#' \item the probability of transition \code{prob}
#' \item and possibly other variables affecting the transition probabilities.
#'   For example \code{soiltype}, \code{treespecies}, etc.
#' }
#' \code{\link{estimatetransprobs}} can be used to estimate transition probabilities
#' from a similar set of pair data of observations.
#'
#' @return An activity definition
#'
#' @param name Name of activity used in reporting
#' @param dynamicvariables Names of variables where changes happen
#' @param transprobs Transition probabilities (\code{data.frame})
#' @param probname (optional) Name of activity in activity probabilities data
#' @examples
#' define_activity("nomanagement", c("vol", "age"))
#' @export
define_activity <- function(name, dynamicvariables, transprobs=NULL, probname=name) {
  act <- list(actname=name,
       actprobname=probname,
       dynamicvariables0=paste0(dynamicvariables, "0", recycle0=TRUE),
       dynamicvariables1=paste0(dynamicvariables, "1", recycle0=TRUE))
  if (!is.null(transprobs))
    efdm::transprobs(act) <- transprobs
  act
}


#' @rdname transprobs
#' @export
`transprobs<-` <- function(act, value) {
  probs <- value
  if(is.null(act$dynamicvariables0) || is.null(act$dynamicvariables1) || is.null(act$actname)) stop("Not a valid activity.")
  if(length(act$dynamicvariables0) == 0)
    stop("Activity without dynamic variables may not have transition probabilities.")
  if(!("prob" %in% names(probs))) stop("probs should have a column named 'prob'")
  vars <- c(act$dynamicvariables1, act$dynamicvariables0)
  if(!all(vars %in% names(probs)))
    stop(paste0("dynamicvariables ", list(setdiff(vars, names(probs)))), " not found in 'probs'")

  aggregatenames <- setdiff(names(probs), c(act$dynamicvariables1, c("prob", "N", "nobs")))
  totalprobs <- aggregate(probs['prob'], probs[aggregatenames], FUN=sum)

  maxdiff <- max(abs(totalprobs$prob-1))
  if(maxdiff > 1e-15) stop(paste0("Not all probabilities sum to 1. Maximum absolute difference ", maxdiff))

  act$A <- probs
  act
}

#' Transition probabilities of an activity
#'
#' Functions to get or set the transition probabilities of an activity
#'
#' The \code{value} should be a \code{data.frame} defining the transition probabilities
#' between dynamic variables of the activity. See \code{\link{define_activity}}
#' for details
#' @param act Activity definition
#' @param value \code{data.frame} of transition probabilities
#' @return \code{data.frame} where prob is the transition probability from current
#' state (with suffix 0) to next state (with suffix 1).
#' @examples
#' act1 <- define_activity("test", c("vol"))
#' transprobs(act1) <- data.frame(vol0 = 1:5, vol1=c(2:5, 5), prob=1)
#' transprobs(act1)
#'
#' if(require("ggplot2")) {
#'   ggplot(transprobs(act1)) + geom_raster(aes(x=vol0, y=vol1, fill=prob))
#' }
#' @export
transprobs <- function(act) {
  act$A
}

library(efdm)
library(testthat)

statespace <- expand.grid(sp=factor(c("spruce", "pine")), vol=1:4)
actprob <- statespace
actprob$test <- actprob$sp == "spruce"
actprob$test2 <- 1-actprob$test

act1 <- define_activity("test", "vol")
transprobs(act1) <- data.frame(sp=factor("spruce"), vol0=1:4, vol1=4, prob=1)

act2 <- define_activity("test2", "vol")
transprobs(act2) <- data.frame(sp=factor("pine"), vol0=1:4, vol1=1, prob=1)

# Factors are matched on their character representation.
# Try to test that anyway
state0 <- data.frame(sp=factor(c("spruce", "pine")), vol=2, area=1)
state1 <- runEFDM(state0, actprob, list(act1, act2), 2)
state0 <- data.frame(sp=factor(c("pine", "spruce")), vol=2, area=1)
state2 <- runEFDM(state0, actprob, list(act1, act2), 2)
expect_equal(state1, state2)

library(efdm)
library(testthat)
orderresult <- function(res) {
  res <- res[do.call(order, res[c("time", "a", "b", "vol", "activity")]),c("time", "a", "b", "vol", "activity", "area")]
  row.names(res) <- NULL
  res
}

statespace <- expand.grid(a=1:2, b=1:2, vol=1:5)
state0 <- statespace
actprob <- statespace
actprob$test <- 1
state0$area <- 0
state0$area[1] <- 1


act1 <- define_activity("test", character())
expect_error(transprobs(act1) <- data.frame(vol0 = 1:4, vol1=2:5, prob=1))
state1 <- runEFDM(state0, actprob, list(act1), 5)
expect_equal(state1$vol, rep(1, 6))

act2 <- define_activity("test2", "vol")
transprobs(act2) <- data.frame(vol0=1:5, vol1=c(2:5,5), prob=1)
actprob$test <- 0.1
actprob$test2 <- 0.9
orderresult(runEFDM(state0, actprob, list(act1, act2), 5))

library(efdm)
library(testthat)
statespace <- expand.grid(a=1:2, b=1:2, vol=1:5)
pairdata <- data.frame(a=c(1,1,2,2), b=c(1,2,1,2), vol0=c(1,1,1,1), vol1=c(2,3,4,5))
state0 <- statespace
actprob <- statespace
actprob$test <- 1
state0$area <- 0
state0$area[1] <- 1

act1 <- define_activity("test", c("vol"))
transprobs(act1) <- estimatetransprobs("vol", pairdata, statespace, by=c("a", "b"), prior="nochange")
A <- transprobs(act1)
res0 <- runEFDM(state0, actprob, list(act1), 1)

transprobs(act1) <- A
res1 <- runEFDM(state0, actprob, list(act1), 1)
expect_identical(res0, res1)

act2 <- define_activity("test", c("vol"))
transprobs(act2) <- A
res2 <- runEFDM(state0, actprob, list(act2), 1)
expect_identical(res0, res2)

A$asdf <- 1
transprobs(act1) <- A
expect_error(runEFDM(state0, actprob, list(act1), 1, check=FALSE),
             "Variable 'asdf' required by activity 'test' not present in state.")
expect_error(runEFDM(state0, actprob, list(act1), 1),
             "Activity 'test' has variable asdf not in actprob.")


act3 <- define_activity("test", c("vol"))
transprobs(act3) <- data.frame(vol0 = 1:5, vol1=c(2:5, 5), prob=1)
res3 <- runEFDM(state0, actprob, list(act3), 3)
# Test something?

library(efdm)
library(testthat)
statespace <- expand.grid(a=1:2, b=1:2, vol=1:5)
state0 <- statespace
actprob <- statespace
actprob$test <- 1
state0$area <- 0
state0$area[1] <- 1


act3 <- define_activity("test", c("vol"))
transprobs(act3) <- data.frame(vol0 = 1:4, vol1=2:5, prob=1)
expect_error(runEFDM(state0, actprob, list(act3), 5), "States with no activity (in list of activities)", fixed=TRUE)
expect_error(runEFDM(state0, data.frame(test=1), list(act3), 5), "No state variables in actprob")



transprobs(act3) <- data.frame(vol0 = 1:5, vol1=c(2:6), prob=1)
expect_error(runEFDM(state0, actprob, list(act3), 3), "Activity transitions to state not in actprob")


expect_error(runEFDM(state0, actprob, list(act3, act3), 3), "States with multiple activities")


act4 <- define_activity("test2", c("vol"))
transprobs(act4) <- data.frame(vol0 = 1:5, vol1=c(2:5,5), prob=1)
expect_error(runEFDM(state0, actprob, list(act4), 5), "Activity probabilities for test2 not given.")

act5 <- define_activity("test", c("vol"))
transprobs(act5) <- data.frame(vol0 = 1:6, vol1=2, prob=1)
expect_error(runEFDM(state0, actprob, list(act5), 5), "Activity transitions from state not in actprob")

act6 <- define_activity("test", c("vol"))
expect_error(runEFDM(state0, actprob, list(act6), 1), "Activity 'test' has no transprobs.")
transprobs(act6) <- data.frame(vol0 = 1:5, vol1=2, c=1, prob=1)
expect_error(runEFDM(state0, actprob, list(act6), 1), "Activity 'test' has variable c not in actprob.")

actprob$test[3] <- NA
expect_error(runEFDM(state0, actprob, list(act3), 5), "Actprob should not have NAs.")

actprob$test2 <- actprob$test <- 0.5
expect_error(runEFDM(state0, actprob, list(act3), 5), "Activity probabilities for test2 given, but activities not.")

library(efdm)
# Test that prior is not dependent on the order of statespace
statespace <- expand.grid(a=1:2, b=1:2, vol=1:15, age=1:35)
statespace <- statespace[sample(nrow(statespace), nrow(statespace)),]

act <- define_activity("test", c("vol", "age"))
act1 <- estimatetransprobs(c("vol", "age"), NULL, statespace, by=c("a", "b"), prior=prior_ff())
A <- transprobs(act1)
stopifnot(all(A$vol1==1 & A$age1 == 1))

act <- define_activity("test", c("vol", "age"))
act1 <- estimatetransprobs(c("vol", "age"), NULL, statespace, factors=c("a", "b"), prior=prior_ff())
A <- transprobs(act1)
stopifnot(all(A$vol1==1 & A$age1 == 1))

act <- define_activity("test", c("vol", "age"))
act1 <- estimatetransprobs(c("vol", "age"), NULL, statespace, factors=c("b"), by="a", prior=prior_ff())
A <- transprobs(act1)
stopifnot(all(A$vol1==1 & A$age1 == 1))

data.table::setDTthreads(1)
library(efdm)

statespace <- expand.grid(ds=c("sp", "pi"), region=c("n", "s"), vol=1:3, stringsAsFactors=FALSE)
statespace$split <- statespace$ds == "sp" & statespace$region == "n"
pairdata <- data.frame(ds=c("sp", "pi"), vol0=c(1,1), vol1=c(2,3), region=c("n", "s"))
pairdata$split <- pairdata$ds == "sp" & pairdata$region == "n"
actprob <- state0 <- statespace
actprob$test <- 1
state0$area <- c(1,1,1,1, 0,0,0,0, 0,0,0,0)

act1 <- define_activity("test", c("vol"))
transprobs(act1) <- estimatetransprobs("vol", pairdata, statespace, factors=c("region"), by=c("split", "ds"), prior="nochange")
r1 <- runEFDM(state0, actprob, list(act1), 10)

statespace <- expand.grid(ds=c("sp", "pi"), region=c("n", "s"), vol=1:3)
statespace$split <- statespace$ds == "sp" & statespace$region == "n"
pairdata <- data.frame(ds=c("sp", "pi"), vol0=c(1,1), vol1=c(2,3), region=c("n", "s"))
pairdata$split <- pairdata$ds == "sp" & pairdata$region == "n"
actprob <- state0 <- statespace
actprob$test <- 1
state0$area <- c(1,1,1,1, 0,0,0,0, 0,0,0,0)

act1 <- define_activity("test", c("vol"))
transprobs(act1) <- estimatetransprobs("vol", pairdata, statespace, factors=c("region"), by=c("split", "ds"), prior="nochange")
r2 <- runEFDM(state0, actprob, list(act1), 10)
r1$ds <- factor(r1$ds, levels=c("sp", "pi"))
r1$region <- factor(r1$region, levels=c("n", "s"))
sortdf <- function(x) x[do.call(order, x),]
stopifnot(all(sortdf(r1)==sortdf(r2)))

#' Example dataset.
#'
#' A list containing the necessary datasets for EFDM forest scenario example.
#'
#' @format A list of data frames:
#' \describe{
#'   \item{actprob}{Activity probabilities}
#'   \item{noman_pairs}{Pair data for growth without management activities}
#'   \item{thin_pairs}{Pair data for thinning}
#'   \item{initial_state}{Initial forest state}
#'   \item{drain_coef}{Coefficient to transform harvested areas into harvest accumulation by timber assortments}
#'   \item{vol_coef}{Coefficients to transform volume classes into volumes m3/ha}
#'   \item{income_coef}{Coefficients to transform harvest accumulation into income}
#' }
"example"

#' Finnish bio-geographical regions
#'
#' A low resolution copy of the finnish bio-geographical regions.
#' The original shapefile was provided by Finnish Environment Institute.
#' See \url{https://ckan.ymparisto.fi/fi/dataset/metsakasvillisuusvyohykkeet}.
#'
#' @format An sf-object:
#' \describe{
#'   \item{region}{Factor with three levels: North, Middle, South}
#'   \item{geometry}{Polygon, each region is composed of multiple polygons}
#' }
"MetsaKasvVyoh"

#' Priors for \code{\link{estimatetransprobs}}
#'
#' Priors for \code{\link{estimatetransprobs}}
#'
#' \code{prior_ff} moves the forest area to the smallest classes of the given
#' dynamic variables of the forest stratum.
#'
#' \code{prior_grow} moves the forest to another class given by increasing
#' \code{variable} by \code{howmuch}.
#'
#' @param variable Name of the variable to grow
#' @param howmuch Amount of growth
#'
#' @return Return value is used by \code{\link{estimatetransprobs}} to provide
#' prior information on the transition probabilities.
#'
#' @examples
#' statespace <- expand.grid(a=1:2, b=1:2, vol=1:15, age=1:15)
#' act <- define_activity("test", c("vol", "age"))
#' act1 <- estimatetransprobs(c("vol", "age"), NULL, statespace, by=c("a", "b"),
#'   prior=prior_ff())
#' act2 <- estimatetransprobs(c("vol", "age"), NULL, statespace, by=c("a", "b"),
#'   prior=prior_grow("age"))
#' @export
prior_ff <- function() {
  function(A, state1, state0) {
    s0 <- expand.grid(sapply(state0, function(x) list(seq_along(x))))

    for(n in names(state1)) {
      s0[[n]] <- 1
    }
    A[as.matrix(s0[c(names(state1), names(state0))])] <- 1
    A
  }
}

#' @rdname prior_ff
#' @export
prior_grow <- function(variable, howmuch=1) {
  function(A, state1, state0) {
    stopifnot(identical(unname(state1), unname(state0)))
    vi <- match(paste0(variable, "1"), names(state1))
    if(is.na(vi)) stop(paste0("Variable '", variable, "' not in statespace. Invalid prior specification."))
    v <- state1[[vi]]
    l <- length(v)
    grid0 <- expand.grid(state1)
    state1[[vi]] <- state1[[vi]][pmax(1, pmin((1:l)+howmuch, l))]
    grid1 <- expand.grid(state1)
    A[as.matrix(cbind(grid1, grid0))] <- 1
    A
  }
}

extract_A <- function(act) {
  A <- act$A
  A$nobs <- NULL
  A$N <- NULL
  A
}

aggregate_area <- function(res, by=names(res)) {
  ns <- setdiff(by, "area")
  r <- as.data.table(res)
  # Using area instead of .SD$area appears to be a lot faster
  area <- stop # This is to trick R Check into not warning about the variable "area"
  as.data.frame(r[,list(area=sum(area)), by=ns])
}

#' @importFrom data.table as.data.table .SD
do_activity <- function(state, act) {
  if(length(act$dynamicvariables0) == 0)
    return(list(before=state, after=state))

  A <- extract_A(act)

  # This check should already be in check_activities, but in case check==FALSE
  requirednames <- c(setdiff(names(A), c("prob", act$dynamicvariables0, act$dynamicvariables1)),
    gsub("0$", "", act$dynamicvariables0))
  missingnames <- setdiff(requirednames, names(state))
  if(length(missingnames) > 0) stop("Variable '", list(missingnames),
                                    "' required by activity '", act$actname, "' not present in state.")


  names1 <- intersect(names(A), names(state))
                        #3c(act$dynamicvariables1, "prob"))
  by.x = c(names1, gsub("0$", "", act$dynamicvariables0))
  by.y = c(names1, act$dynamicvariables0)
  res <- merge(as.data.table(state), as.data.table(A), by.x=by.x, by.y=by.y,
               all.x=FALSE, all.y=FALSE, allow.cartesian=TRUE)
  res$area <- res$area * res$prob
  res$prob <- NULL
  resbefore <- res
  for(n in act$dynamicvariables0)
    res[[gsub("0$", "", n)]] <- NULL
  for(n in act$dynamicvariables1) {
    i <- match(n, names(res))
    names(res)[i] <- gsub("1$", "", names(res)[i])
    resbefore[[n]] <- NULL
  }
  resbefore <- aggregate_area(resbefore)
  res <- aggregate_area(res)

  if(!isTRUE(all.equal(sum(resbefore$area), sum(res$area)))) {
    stop(paste0("Some area (", sum(resbefore$area), "!=", sum(res$area), ") was lost in the computation for act ", act$actname))
  }
  list(before=resbefore, after=res)
}

check_activities <- function(acts, actprob, actnames, actfactors) {
  actnames1 <- sapply(acts, function(x) x$actprobname)
  if(!all(actnames1 %in% actnames)) stop(paste0("Activity probabilities for ", setdiff(actnames1, actnames), " not given."))
  if(!all(actnames %in% actnames1)) stop(paste0("Activity probabilities for ", setdiff(actnames, actnames1), " given, but activities not."))

  P <- actprob
  P$check___ <- 0
  if(anyNA(P)) stop("Actprob should not have NAs.")

  if(any(P[actnames] < 0)) stop("Activity probabilities should be positive.")

  maxdiff <- max(abs(rowSums(P[actnames])-1))
  if(maxdiff > 1e-15) stop(paste0("Not all activity probabilities sum to 1. Maximum absolute difference ", maxdiff))


  # Check states before transition
  for(act in acts) {
    A <- extract_A(act)

    if(is.null(A) && length(act$dynamicvariables0) != 0)
      stop(paste0("Activity '", act$actname, "' has no transprobs."))

    for(v in act$dynamicvariables1)
      A[[v]] <- NULL
    for(v0 in act$dynamicvariables0) {
      v <- gsub("0$", "", v0)
      A[[v]] <- A[[v0]]
      A[[v0]] <- NULL
    }

    A$prob <- NULL

    if(!(all(names(A) %in% names(P))))
      stop(paste0("Activity '", act$actname, "' has variable ", list(setdiff(names(A), names(P))), " not in actprob."))

    A <- A[!duplicated(A),,drop=FALSE]
    A$check_new__ <- 1
    P <- merge(P, A, all.x=TRUE, all.y=TRUE)
    P$check___ <- P$check___ + ifelse(is.na(P$check_new__), 0, P[[act$actname]])
    P$check_new__ <- NULL
  }
  check <- is.na(P$check___)
  if(any(check)) {
    print("Activity transitions from state not in actprob:")
    print(utils::head(P[check,,drop=FALSE]))
    stop("Activity transitions from state not in actprob")
  }
  check <- P$check___ == 0
  if(any(check)) {
    print("States with no activity (in list of activities):")
    print(utils::head(P[check,,drop=FALSE]))
    stop("States with no activity (in list of activities)")
  }
  P$check_1__ <- P$check___ - 1
  check <- abs(P$check_1__) > 1e-14
  if(any(check)) {
    print("States with multiple activities:")
    print(utils::head(P[check,,drop=FALSE]))
    stop("States with multiple activities")
  }

  # Check states after transition
  for(act in acts) {
    if(length(act$dynamicvariables0) > 0) {
      A <- extract_A(act)
      for(v in act$dynamicvariables0)
        A[[v]] <- NULL
      for(v1 in act$dynamicvariables1) {
        v <- gsub("1$", "", v1)
        A[[v]] <- A[[v1]]
        A[[v1]] <- NULL
      }
      A$prob <- NULL
      A <- A[!duplicated(A),,drop=FALSE]
      P <- merge(P, A, all.x=TRUE, all.y=TRUE)
      check <- is.na(P$check___)
      if(any(check)) {
        print(paste0("Activity '", act$actname, "' caused transitions to state not in actprob:"))
        print(utils::head(P[check,,drop=FALSE]))
        stop("Activity transitions to state not in actprob")
      }
    }
  }
}



#' Run European Forestry Dynamics Model
#'
#' Run European Forestry Dynamics Model
#'
#' This is the actual scenario running function, which projects the initial
#' forest state n time steps to the future.
#'
#' Activities are defined using \code{\link{define_activity}}.
#'
#'
#' @param state0 \code{data.frame} Initial state
#' @param actprob \code{data.frame} Activity probabilities
#' @param activities \code{list} A list of activities
#' @param n \code{integer} Number of time steps required
#' @param check Check input arguments for consistency.
#' @param format ??
#' @return \code{data.frame} State of each time step divided by activities
#' @importFrom stats aggregate
#' @importFrom utils head
#' @export
runEFDM <- function(state0, actprob, activities, n, check=TRUE, format=1) {
  totalarea <- sum(state0$area)
  state <- state0
  # state may include for example plot id which should not be anywhere else
  actfactors <- intersect(names(state), names(actprob))
  actnames <- setdiff(names(actprob), names(state))

  if(length(actfactors)==0) stop("No state variables in actprob.")

  if(check) {
    if(anyDuplicated(actprob[setdiff(names(actprob), actnames)]))
      stop("Duplicated rows in actprob.")

    check_activities(activities, actprob, actnames, actfactors)
  }
  # Later checks should not be needed, but they are cheap and might be useful if check==FALSE

  beforeactivity <- NULL
  for(i in 0:ifelse(format==1, n, n-1)) {
    m <- as.data.frame(merge(as.data.table(state), as.data.table(actprob), by=actfactors, all.x=TRUE, all.y=FALSE, allow.cartesian=TRUE))

    if(any(is.na(m[actnames]))) {
      print("No activation probability for:")
      print(head(m[is.na(m[length(m)]),]))
      print("...")
      stop("Stopping")
    }
    checkareain <- list()
    checkareaout <- list()
    for(actname in actnames) {
      checkareaout[[actname]] <- 0
    }

    res <- list()
    for(act in activities) {
      m2 <- m
      m2$area <- m2$area*m2[[act$actprobname]]
      m2 <- m2[names(state)]
      m2 <- m2[m2$area != 0,, drop=FALSE]
      checkareain[[act$actprobname]] <- sum(m2$area)
      # m2 can include some area that is not meant for this activity due to
      # activities having different statespace but shared activation probability
      if(nrow(m2) != 0) {
        res2 <- do_activity(m2, act)
        res1 <- res2$after
        if(nrow(res1) != 0)  {
          extranames <- setdiff(names(state), c(names(res1), "area"))
          if(length(extranames) != 0) {
            stop(paste0("Variable '", list(extranames), "' in state before activity '", act$actname, "' but not after."))
          }
          res1$activity <- act$actname
          res1$time <- i
          res <- c(res, list(res1))
          checkareaout[[act$actprobname]] <- checkareaout[[act$actprobname]] + sum(res1$area)
        }
        res1 <- res2$before
        res1 <- res1[!is.na(res1$area),,drop=FALSE]
        if(nrow(res1) != 0)  {
          res1$activity <- act$actname
          res1$time <- i
          beforeactivity <- c(beforeactivity, list(res1))
        }
      }
    }

    for(actname in actnames) {
      if(!isTRUE(all.equal(checkareain[[actname]], checkareaout[[actname]])))
        stop(paste0("Activity '", actname, "' was allocated ", checkareain[[actname]], " area, but it processed ", checkareaout[[actname]], "."))
    }

    if(length(res) == 0) {
      stop(paste0("All area was lost after time ", i, "."))
    }
    allres <- do.call(rbind, res)
    state <- aggregate_area(allres, names(state))
    newtotalarea <- sum(state$area)
    if(!isTRUE(all.equal(totalarea, newtotalarea)))
      warning(paste("Starting with", totalarea, "area ended up with", newtotalarea, " area."))
  }
  beforeactivity <- do.call(rbind, beforeactivity)
  if(format == 2) {
    return(list(beforeactivity=beforeactivity, stateend=state))
  }
  beforeactivity
}

# What was this??
data.table::setDTthreads(1)
library(efdm)
statespace <- expand.grid(a=1:2, b=1:2, vol=1:5)
pairdata <- data.frame(a=c(1,1,2,2), b=c(1,2,1,2), vol0=c(1,1,1,1), vol1=c(2,3,4,5))
state0 <- statespace
actprob <- statespace
actprob$test <- 1
state0$area <- 0
state0$area[1] <- 1

prior <- statespace
prior$vol0 <- prior$vol1 <- prior$vol
prior$vol <- NULL
prior$nobs <- 3
if(FALSE) {
  act4 <- define_activity("test", c("vol"))
  transprobs(act4) <- estimatetransprobs("vol", pairdata, statespace, factors=c("a", "b"), prior=prior)
  transprobs(act4)
  runEFDM(state0, actprob, list(act4), 1)
  act1 <- define_activity("test", c("vol"))
  transprobs(act1) <- estimatetransprobs("vol", pairdata, statespace, by=c("a", "b"), prior=prior)
  transprobs(act1)
  runEFDM(state0, actprob, list(act1), 1)
}

---
title: "Example scenarios"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 6
vignette: >
  %\VignetteIndexEntry{Example scenarios}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
options(max.print=50)
data.table::setDTthreads(1)
```


# Introduction

European Forestry Dynamics Model (EFDM) is a model for large scale scenario analysis. The model is an area-based matrix model where forest area is stratified with respect to, for example, use, management strategy  and development (growth). The model can consider also other land uses, see Scenario 3. In a model run forest area moves in 'a matrix' span with the stratifying factors.

The `efdm` package includes functions to

  - define management activities
  - estimate transition probabilities
  - run an EFDM scenario

and an example dataset for illustration purposes.

In this vignette, we demonstrate 'efdm' in running a scenario with the example dataset.
Typical workflow in a EFDM scenario analysis starts with preparing the datasets required as an input. Preparing the datasets is usually the most time consuming part of the scenario analysis, and the work flow depends on the data available. Therefore, the preparation is not explained in this vignette. However, the attached datasets offer an example for preparing own datasets.


The `efdm` is loaded with
```{r}
library(efdm)
```

The following packages are used in this vignette to manipulate data and plot results.
However, the data manipulation and reporting the results could be done using other packages or even outside the R environment.
```{r setup, warning = FALSE, message = FALSE}
library(dplyr)
library(ggplot2)
library(sf)
```


# Example data

In this vignette we are using `example`, a collection of example datasets to
describe working with `efdm`. The dataset covers entire Finland and the initial state represents year 2016.
However, the dataset is a toy data which is meant for learning the model concept.
```{r}
data(example)
```

In this example we have chosen

  - a 5-year simulation step
  - age and total growing stock volume per hectare to present forest state
	- age is classified into 5 year classes, starting from class 1 = 0-5 years, and ending at class 35 = 170+ years with all forests over 170 years old
	- volume is classified into 15 classes with class limits depending on the dominant tree species and their growth pattern
  - additional stratification variables defining different forest strata are spatial region (North, Middle, South), soil type (Mineral, Peat) and dominant tree species (spruce, other species)

# Activities

In EFDM forest management is represented by activities and activity probabilities.
In the first example we have three activities

 - thinning (thin)
 - final felling (ff)
 - no management (noman)

## Activity probability and transition probabilities

The activity probabilities (`actprob` below) are the probabilities for applying a specific
management activity in a specific forest state described by the stratification variables.
The activity probabilities are used to allocate forest area to different activities.
A valid activity probability specifies activity probabilities for all possible states and activities.
All probabilities should be positive and the sum of activity probabilities for each state should be 1.
```{r}
actprob <- example$actprob
actprob
```

Each activity has a transition probability matrix.
The transition probability describes movements of forest areas by the simulation time steps.
It is used to move a forest state to the next states.
A valid transition probability should specify transition probability for each state where the activity can be used.
For example if final felling only applies to forests with some minimum volume, then there is no need to specify transitions from states with less than the minimum volume.

## Final felling

The final felling activity is represented by a transition matrix that has
probability 1 to move any forest to a state where volume and age are in
the smallest class.

The target state is independent of the starting state. It will always be `vol=1` and `age=1`, the smallest class in the stratification.
```{r}
transprobs_ff <- expand.grid(vol0=1:15, age0=1:35, vol1=1, age1=1, prob=1)
transprobs_ff
```
In `efdm` the final felling activity is defined by giving

  - name of the activity, "ff"
  - names of the dynamic variables, that is the variables that the final felling activity affects, "vol" and "age"
  - transition probabilities

```{r ff}
ff <- define_activity("ff", c("vol", "age"), transprobs_ff)
```


## No management

If there are no management activities or treatments in the forest, "no management" activity is
used to make the forest grow. In this example, we assume that growth changes the
values of volume and age and is dependent on region, soil type and tree species.
Moreover, we assume that the growth is so different in between regions and tree
species, that it should be estimates completely separately for each class.

### Estimation of transition matrix using pair data

To estimate the transition matrix, we use "pair data" obtained from two
consecutive measurements of permanent sample plots in the national forest inventory.
For `estimatetransprobs`, the estimation function implemented
in `efdm`, the pair data is a `data.frame` of pairs of dynamic variables and
possible stratification variables, which affect the growth but are not affected
by growth.
```{r pairdata}
example$noman_pairs
```

In order to use `estimatetransprobs` we still need two parts, a state space and
a prior.
The state space and prior are required because the pair data might not have observations
for all possible states, but the transition matrix should be available for all
states.

#### State space

The state space is the collection of all possible states of stratifying variables.
Since we have an activity probability for each state we can use the `actprob` to
obtain the `statespace`

```{r statespace}
statespace <- actprob %>% select(c(region, soil, sp, vol, age))
statespace
```
In this example dataset the number of states in the state space is the product of the number of classes in each stratifying variable, that is $3*2*2*15*35 = 6300$.

#### Prior

The prior in `estimatetransprobs` works by adding one observation to the pair data
for each starting state.
The `efdm` has a few standard prior choices.

  - `prior_grow(variable)` grows `variable` by one class.
  - `"nochange"`, forest state is not changed
  - `"uninformative"`, an observation with tiny weight is added to every possible
  target state.

For the no management activity we are using `prior_grow("age")` stating
that there is always at least one observation where the age grows by one class
but the volume is not changing.


#### Estimation

Now we are ready to estimate the transition probabilities for no management
```{r}
transprobs_noman <- estimatetransprobs(c("vol", "age"), # the dynamic variables of the activity
                     example$noman_pairs,  # pair data for no management
                     statespace=statespace,
                     factors=c("soil"), # Information for different soil types
                                        # is used with smaller weight
                     by=c("region","sp"), # Separate estimation for each
                                          # region and species
                     prior=prior_grow("age"))
```
and to define the activity `noman`
```{r noman}
noman <- define_activity("noman", c("vol", "age"), transprobs_noman)
```

The resulting transition probabilities look like
```{r}
transprobs_noman %>%
  arrange(vol0, age0, soil, region, sp, vol1, age1)
```

## Thinning

We use exactly the same procedure for thinning that we used for no management.
The only differences are the activity name and the pair data used.

```{r thin}
transprobs_thin <- estimatetransprobs(c("vol", "age"),
                     example$thin_pairs, # pair data for thinning
                     statespace=statespace,
                     factors=c("soil"),
                     by=c("region","sp"),
                     prior=prior_grow("age"))
thin <- define_activity("thin", c("vol", "age"), transprobs_thin)
```


# Scenario 1

EFDM is an area based model. The initial state of the scenario is a `data.frame`
where for each state there is an `area` (zeros may be omitted)
```{r}
state0 <- example$initial_state
state0
```

Next, we run the EFDM scenario for 20 time steps (100 years) starting with
initial state `state0`, activity probabilities `actprob` and a list of activities.
```{r}
activities <- list(noman, thin, ff)
states1 <- runEFDM(state0, actprob, activities, 20)
```

`runEFDM` produces a `data.frame` of areas allocated to each activity at each time step.

```{r}
states1 %>%
  arrange(soil, region, sp, vol, age, time, activity)
```

## Analysing results

### Result coefficients

Using so called result coefficients is a way to obtain results with respect to other forest properties than area.
In this scenario we have growing stock volume, drain and harvest income as result variables.

The volume coefficients convert volume class `vol` and dominant species `sp` into growing stock volume `volume` (m3/ha) in four tree species groups (`species`): pine, spruce, broadleaves and all.
The volume coefficients were estimated for the classes based on the species composition in the forest inventory data as class averages.

```{r}
example$vol_coef %>%
  arrange(vol, sp, species)
```

Drain is the harvest accumulation which is linked to management activities (activity), and separated by the volume class (vol) and dominant species (sp). It was also estimated from the observed changes in the forest inventory data as class averages.
The drain is given as m3 per ha (drain) and timber assortment (assort): pulp/saw wood.
```{r}
example$drain_coef %>%
  arrange(-vol, sp, assort, activity)
```

Income is loosely based on the actual statistics of Finnish timber assortment prices over last years in unit eur/m3.
```{r}
example$income_coef
```

### Results
Simulation timesteps are mutated to mid-years of simulation steps (the simulation begins from year 2016):
```{r}
states1 <- states1 %>% mutate(time = factor(2016 + time*5))
```


Growing stock volume is estimated by merging the area distribution by simulation steps (states1) and volume coefficients. The total growing stock volume (m3) is a result of multiplication of area (ha) and coefficient (m3/ha). Finally the result is visualized in a figure.
```{r}
volume <- merge(states1, example$vol_coef) %>%
  mutate (volume=area*volume) %>% #area is multiplied with m3/ha volume the get total volume in m3
  filter(species != "all")
ggplot(volume) +
  scale_fill_viridis_d(end = 0.9) +
  geom_bar(aes(x=time, weight=volume/1000000000, fill=species)) +
  labs(y=NULL,title=expression(paste("Growing stock, bil.",m^3)), x="Year", fill="") +
  theme(legend.position = "bottom", axis.text.x = element_text(angle = -90))
```

Estimating age distribution does not require result coefficients. However, the age classes are defined for 50-year classes instead of original 5-year classes, and then plotted for only three time steps: years 2016, 2066 and 2116.
```{r}
states1$ageclass <- cut(states1$age, breaks=c(0,10,20,30,35), include.lowest = TRUE, 
                        #labels=c("0-50","51-100","101-150","150+"))
                        labels=c("-50","-100","-150","150+"))
states1$region <- factor(states1$region, labels = c("South", "Middle", "North"))
ggplot(subset(states1, time %in% c(2016,2066,2116))) +
  geom_bar(aes(x=ageclass, weight=area/1000000, fill=region)) +  
  scale_fill_viridis_d(end = 0.9) +
  facet_grid(cols=vars(time)) +
  labs(y=NULL, title="Area, mill.ha", x="Ageclass", fill=NULL) +
  theme(legend.position = "bottom", axis.text.x = element_text(angle = -90))
```


Income is estimated by first converting the drain (m3/ha) into income (eur/ha). Then multiplication with area (ha) gives the euros.
```{r}
euro <- merge(example$drain_coef, example$income_coef) %>% mutate(euro = euro*drain)
removal <- merge(states1, euro) %>% mutate(income=euro*area)

ggplot(subset(removal,!time %in% c(2116))) + 
  geom_bar(aes(x=time, weight=income/5000000000, fill=assort)) +
  scale_fill_viridis_d(end = 0.9) +
  labs(y=NULL,title = "Income, bil.€/year", x="5-year intervals", fill=NULL ) +
  theme(legend.position = "bottom", axis.text.x = element_text(angle = -90))
```





# Scenario 2
In this example the tree species changes after final felling. Therefore we redefine final felling activity taking into account in addition to volume and age also the dominant species. The change probability depends on region and dominant species.
Volume and age act as before. They move to the smallest classes (vol1=1 and age1=1).
```{r}
transprobs_ff_age_species <- statespace %>%
  select(vol0=vol, age0=age, sp0=sp, region) %>%
  unique %>%
  group_by(region, sp0, vol0, age0) %>%
  summarize(data.frame(vol1=1, age1=1, sp1=c('other','spruce'),
                       prob=case_when(
                         sp0=='other' && region=='South' ~ c(1, 0),
                         sp0=='other' && region=='Middle' ~ c(1, 0),
                         sp0=='other' && region=='North' ~ c(0.8, 0.2), #in other dominated forest
                                        # in northern boreal vegetation zone 20% of final felled area
                                        # change to spruce dominated forest
                         sp0=='spruce' && region=='South' ~ c(0.3, 0.7),
                         sp0=='spruce' && region=='Middle' ~ c(0.2, 0.8),
                         sp0=='spruce' && region=='North' ~ c(0, 1))))
ff_age_species <- define_activity("ff", c("vol", "age", "sp"), transprobs_ff_age_species)
```

The list of activities includes the same noman and thin as in Scenario 1, and the above defined final felling. The same initial state as in Scenario 1 is run with the new list of activities and stored in variable states2.
```{r}
activities2 <- list(noman, thin, ff_age_species)
states2 <- runEFDM(state0, actprob, activities2, 20)
```


The proportion of spruce dominated forest area is estimated for time steps 0, 10 and 20 (years 2016, 2066 and 2116). And the result is presented by the vegetation zones as a map.

```{r}
# Compute proportion of spruces in each time and region
prop_spruces <- states2 %>%
  group_by(region, time) %>%
  summarise(proportion = 100*sum((sp=="spruce")*area)/sum(area)) %>%
  filter(time %in% c(0, 10, 20)) %>%
  mutate(time = 2016 + 5*time)

# Add bio-geographical regions (MetsaKasvVyoh provided in efdm package) to the data
prop_spruces <- merge(MetsaKasvVyoh, prop_spruces)
prop_spruces %>% ggplot() +
  geom_sf(aes(fill=proportion)) +
  facet_wrap(vars(time)) +
  scale_fill_gradient(low = "white", high = "forestgreen") +
  labs(fill="Proportion" ,title="Proportion of spruce dominated forests, %") +
  guides(fill = guide_colourbar(frame.colour = "grey20",ticks.colour = "grey20")) +
  theme(legend.position = "bottom", axis.text.x = element_text(angle = -90))
```

The map provided by: Finnish Environment Institute

# Scenario 3 - Land use changes

First we add two land use classes to the statespace. Agriculture is stratified according to region and soil type, while other land use is only stratified according to region.
```{r}
statespace3 <- statespace
statespace3$landuse <- "forest"
agriculture <- unique(statespace3 %>% mutate(sp=0, vol=0, age=0, landuse="agriculture"))
other <- unique(statespace3 %>% mutate(soil=0, sp=0, vol=0, age=0, landuse="other"))
statespace3 <- rbind(statespace3, agriculture, other)
```

We use separate activities for deforestation to each land use class.
Variables (vol, age, sp) not used by the agriculture are set to 0.
Soil type and region are not changing as a result of deforestation to agriculture.
```{r}
transprobs_defor_to_agri <- statespace3 %>%
  filter(landuse=="forest") %>%
  select(vol0=vol, age0=age, sp0=sp,landuse0=landuse) %>%
  unique %>%
  mutate(vol1=0, age1=0, sp1=0, landuse1="agriculture", prob=1)
defor_to_agri <- define_activity("defor_to_agri",
                                 c("vol", "age", "sp", "landuse"),
                                 transprobs_defor_to_agri)
```

Deforestation to other land use also changes soil type to 0.
```{r}

transprobs_defor_to_other <- statespace3 %>%
  filter(landuse=="forest") %>%
  select(soil0=soil, vol0=vol, age0=age, sp0=sp,landuse0=landuse) %>%
  unique %>%
  mutate(soil1=0, vol1=0, age1=0, sp1=0, landuse1="other", prob=1)
defor_to_other <- define_activity("defor_to_other",
                                  c("soil", "vol", "age", "sp", "landuse"),
                                  transprobs_defor_to_other)
```

Afforestation only applies to agriculture.
Volume and age classes start from 1 and the area is split evenly to spruce and other species.
```{r}
transprobs_aff <- statespace3 %>%
  filter(landuse=="agriculture") %>%
  select(soil, vol0=vol, age0=age, region, sp0=sp, landuse0=landuse) %>%
  mutate(vol1=1, age1=1, sp1="other", landuse1="forest", prob=1)
transprobs_aff <- rbind(transprobs_aff %>% mutate(sp1="spruce", prob=0.5),
                        transprobs_aff %>% mutate(sp1="other", prob=0.5))
affor <- define_activity("affor",
                         c("vol", "age", "sp", "landuse"),
                         transprobs_aff)
```

A donothing activity is used for non-forest land uses, when there is nothing forestry related going on.
```{r}
donothing <- define_activity("donothing", character())
activities3 <- list(noman, thin, ff, defor_to_other, defor_to_agri, affor, donothing)
```

adding land use information to state 
```{r}
state03 <- state0
state03$landuse <- "forest"
state03 <- rbind(state03,  agriculture %>% mutate(area=rep(c(1552000/2,997000/2,76000/2),each=2)),
                 other %>% mutate(area=c(721000,507000,112000)))
```

Activity probabilities for new land uses 
```{r}
actprob3 <- actprob
actprob3$landuse <- "forest"
# Define probabilities for new activities
actprob3$defor_to_other <- 0.0002
actprob3$defor_to_agri <- 0.00025
actprob3$affor <- 0
actprob3$donothing <- 0
# Normalise probabilities after adding new activities
actnames <- c("noman", "thin", "ff", "defor_to_other", "defor_to_agri", "affor", "donothing")
actprob3[actnames] <- actprob3[actnames]/rowSums(actprob3[actnames])
# Define activity probabilities also for agriculture and other land uses
actprob3 <- rbind(actprob3, 
        agriculture %>% mutate(thin=0, ff=0, noman=0, defor_to_other=0, defor_to_agri=0, affor=0.0005, donothing=0.9995),
        other %>% mutate(thin=0, ff=0, noman=0, defor_to_other=0, defor_to_agri=0, affor=0, donothing=1))
```


```{r}
states3 <- runEFDM(state03, actprob3, activities3, 20)
```

```{r}
#pie charts of land use areas
LUareas <- states3 %>%
  group_by(time, landuse) %>%
  summarise(area = sum(area)) %>%
  ungroup()
LUareas <- LUareas %>%
  mutate(time = factor(time, labels = seq(2016,2116,5)))
totalarea <- sum(state03$area)
LUareas %>% filter(time %in% c("2016","2066","2116")) %>%
  mutate(area = area/totalarea) %>%
  ggplot() +
  geom_bar(aes(x="",y=area, fill=landuse), stat="identity", width=1, color = "white") +
  scale_fill_viridis_d(end = 0.9) +
  coord_polar("y", start=0) +
  facet_wrap(vars(time)) +
  labs(y=NULL,fill="Land use", x=NULL)+
  theme(axis.text.x = element_blank(),
        axis.ticks = element_blank()) +
  geom_text(aes(x="",y = rep(c(0.03, 0.33, 0.95), 3), label = round(area*100,1)),
            nudge_x=0.25, size=2.25,
            color=rep(c("grey20","white","white"), 3), fontface="bold") +
  theme(legend.position = "bottom")
```

---
title: "Example scenarios"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 6
vignette: >
  %\VignetteIndexEntry{Example scenarios}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
options(max.print=50)
data.table::setDTthreads(1)
```


# Introduction

European Forestry Dynamics Model (EFDM) is a model for large scale scenario analysis. The model is an area-based matrix model where forest area is stratified with respect to, for example, use, management strategy  and development (growth). The model can consider also other land uses, see Scenario 3. In a model run forest area moves in 'a matrix' span with the stratifying factors.

The `efdm` package includes functions to

  - define management activities
  - estimate transition probabilities
  - run an EFDM scenario

and an example dataset for illustration purposes.

In this vignette, we demonstrate 'efdm' in running a scenario with the example dataset.
Typical workflow in a EFDM scenario analysis starts with preparing the datasets required as an input. Preparing the datasets is usually the most time consuming part of the scenario analysis, and the work flow depends on the data available. Therefore, the preparation is not explained in this vignette. However, the attached datasets offer an example for preparing own datasets.


The `efdm` is loaded with
```{r}
library(efdm)
```

The following packages are used in this vignette to manipulate data and plot results.
However, the data manipulation and reporting the results could be done using other packages or even outside the R environment.
```{r setup, warning = FALSE, message = FALSE}
library(dplyr)
library(ggplot2)
library(sf)
```


# Example data

In this vignette we are using `example`, a collection of example datasets to
describe working with `efdm`. The dataset covers entire Finland and the initial state represents year 2016.
However, the dataset is a toy data which is meant for learning the model concept.
```{r}
data(example)
```

In this example we have chosen

  - a 5-year simulation step
  - age and total growing stock volume per hectare to present forest state
	- age is classified into 5 year classes, starting from class 1 = 0-5 years, and ending at class 35 = 170+ years with all forests over 170 years old
	- volume is classified into 15 classes with class limits depending on the dominant tree species and their growth pattern
  - additional stratification variables defining different forest strata are spatial region (North, Middle, South), soil type (Mineral, Peat) and dominant tree species (spruce, other species)

# Activities

In EFDM forest management is represented by activities and activity probabilities.
In the first example we have three activities

 - thinning (thin)
 - final felling (ff)
 - no management (noman)

## Activity probability and transition probabilities

The activity probabilities (`actprob` below) are the probabilities for applying a specific
management activity in a specific forest state described by the stratification variables.
The activity probabilities are used to allocate forest area to different activities.
A valid activity probability specifies activity probabilities for all possible states and activities.
All probabilities should be positive and the sum of activity probabilities for each state should be 1.
```{r}
actprob <- example$actprob
actprob
```

Each activity has a transition probability matrix.
The transition probability describes movements of forest areas by the simulation time steps.
It is used to move a forest state to the next states.
A valid transition probability should specify transition probability for each state where the activity can be used.
For example if final felling only applies to forests with some minimum volume, then there is no need to specify transitions from states with less than the minimum volume.

## Final felling

The final felling activity is represented by a transition matrix that has
probability 1 to move any forest to a state where volume and age are in
the smallest class.

The target state is independent of the starting state. It will always be `vol=1` and `age=1`, the smallest class in the stratification.
```{r}
transprobs_ff <- expand.grid(vol0=1:15, age0=1:35, vol1=1, age1=1, prob=1)
transprobs_ff
```
In `efdm` the final felling activity is defined by giving

  - name of the activity, "ff"
  - names of the dynamic variables, that is the variables that the final felling activity affects, "vol" and "age"
  - transition probabilities

```{r ff}
ff <- define_activity("ff", c("vol", "age"), transprobs_ff)
```


## No management

If there are no management activities or treatments in the forest, "no management" activity is
used to make the forest grow. In this example, we assume that growth changes the
values of volume and age and is dependent on region, soil type and tree species.
Moreover, we assume that the growth is so different in between regions and tree
species, that it should be estimates completely separately for each class.

### Estimation of transition matrix using pair data

To estimate the transition matrix, we use "pair data" obtained from two
consecutive measurements of permanent sample plots in the national forest inventory.
For `estimatetransprobs`, the estimation function implemented
in `efdm`, the pair data is a `data.frame` of pairs of dynamic variables and
possible stratification variables, which affect the growth but are not affected
by growth.
```{r pairdata}
example$noman_pairs
```

In order to use `estimatetransprobs` we still need two parts, a state space and
a prior.
The state space and prior are required because the pair data might not have observations
for all possible states, but the transition matrix should be available for all
states.

#### State space

The state space is the collection of all possible states of stratifying variables.
Since we have an activity probability for each state we can use the `actprob` to
obtain the `statespace`

```{r statespace}
statespace <- actprob %>% select(c(region, soil, sp, vol, age))
statespace
```
In this example dataset the number of states in the state space is the product of the number of classes in each stratifying variable, that is $3*2*2*15*35 = 6300$.

#### Prior

The prior in `estimatetransprobs` works by adding one observation to the pair data
for each starting state.
The `efdm` has a few standard prior choices.

  - `prior_grow(variable)` grows `variable` by one class.
  - `"nochange"`, forest state is not changed
  - `"uninformative"`, an observation with tiny weight is added to every possible
  target state.

For the no management activity we are using `prior_grow("age")` stating
that there is always at least one observation where the age grows by one class
but the volume is not changing.


#### Estimation

Now we are ready to estimate the transition probabilities for no management
```{r}
transprobs_noman <- estimatetransprobs(c("vol", "age"), # the dynamic variables of the activity
                     example$noman_pairs,  # pair data for no management
                     statespace=statespace,
                     factors=c("soil"), # Information for different soil types
                                        # is used with smaller weight
                     by=c("region","sp"), # Separate estimation for each
                                          # region and species
                     prior=prior_grow("age"))
```
and to define the activity `noman`
```{r noman}
noman <- define_activity("noman", c("vol", "age"), transprobs_noman)
```

The resulting transition probabilities look like
```{r}
transprobs_noman %>%
  arrange(vol0, age0, soil, region, sp, vol1, age1)
```

## Thinning

We use exactly the same procedure for thinning that we used for no management.
The only differences are the activity name and the pair data used.

```{r thin}
transprobs_thin <- estimatetransprobs(c("vol", "age"),
                     example$thin_pairs, # pair data for thinning
                     statespace=statespace,
                     factors=c("soil"),
                     by=c("region","sp"),
                     prior=prior_grow("age"))
thin <- define_activity("thin", c("vol", "age"), transprobs_thin)
```


# Scenario 1

EFDM is an area based model. The initial state of the scenario is a `data.frame`
where for each state there is an `area` (zeros may be omitted)
```{r}
state0 <- example$initial_state
state0
```

Next, we run the EFDM scenario for 20 time steps (100 years) starting with
initial state `state0`, activity probabilities `actprob` and a list of activities.
```{r}
activities <- list(noman, thin, ff)
states1 <- runEFDM(state0, actprob, activities, 20)
```

`runEFDM` produces a `data.frame` of areas allocated to each activity at each time step.

```{r}
states1 %>%
  arrange(soil, region, sp, vol, age, time, activity)
```

## Analysing results

### Result coefficients

Using so called result coefficients is a way to obtain results with respect to other forest properties than area.
In this scenario we have growing stock volume, drain and harvest income as result variables.

The volume coefficients convert volume class `vol` and dominant species `sp` into growing stock volume `volume` (m3/ha) in four tree species groups (`species`): pine, spruce, broadleaves and all.
The volume coefficients were estimated for the classes based on the species composition in the forest inventory data as class averages.

```{r}
example$vol_coef %>%
  arrange(vol, sp, species)
```

Drain is the harvest accumulation which is linked to management activities (activity), and separated by the volume class (vol) and dominant species (sp). It was also estimated from the observed changes in the forest inventory data as class averages.
The drain is given as m3 per ha (drain) and timber assortment (assort): pulp/saw wood.
```{r}
example$drain_coef %>%
  arrange(-vol, sp, assort, activity)
```

Income is loosely based on the actual statistics of Finnish timber assortment prices over last years in unit eur/m3.
```{r}
example$income_coef
```

### Results
Simulation timesteps are mutated to mid-years of simulation steps (the simulation begins from year 2016):
```{r}
states1 <- states1 %>% mutate(time = factor(2016 + time*5))
```


Growing stock volume is estimated by merging the area distribution by simulation steps (states1) and volume coefficients. The total growing stock volume (m3) is a result of multiplication of area (ha) and coefficient (m3/ha). Finally the result is visualized in a figure.
```{r}
volume <- merge(states1, example$vol_coef) %>%
  mutate (volume=area*volume) %>% #area is multiplied with m3/ha volume the get total volume in m3
  filter(species != "all")
ggplot(volume) +
  scale_fill_viridis_d(end = 0.9) +
  geom_bar(aes(x=time, weight=volume/1000000000, fill=species)) +
  labs(y=NULL,title=expression(paste("Growing stock, bil.",m^3)), x="Year", fill="") +
  theme(legend.position = "bottom", axis.text.x = element_text(angle = -90))
```

Estimating age distribution does not require result coefficients. However, the age classes are defined for 50-year classes instead of original 5-year classes, and then plotted for only three time steps: years 2016, 2066 and 2116.
```{r}
states1$ageclass <- cut(states1$age, breaks=c(0,10,20,30,35), include.lowest = TRUE, 
                        #labels=c("0-50","51-100","101-150","150+"))
                        labels=c("-50","-100","-150","150+"))
states1$region <- factor(states1$region, labels = c("South", "Middle", "North"))
ggplot(subset(states1, time %in% c(2016,2066,2116))) +
  geom_bar(aes(x=ageclass, weight=area/1000000, fill=region)) +  
  scale_fill_viridis_d(end = 0.9) +
  facet_grid(cols=vars(time)) +
  labs(y=NULL, title="Area, mill.ha", x="Ageclass", fill=NULL) +
  theme(legend.position = "bottom", axis.text.x = element_text(angle = -90))
```


Income is estimated by first converting the drain (m3/ha) into income (eur/ha). Then multiplication with area (ha) gives the euros.
```{r}
euro <- merge(example$drain_coef, example$income_coef) %>% mutate(euro = euro*drain)
removal <- merge(states1, euro) %>% mutate(income=euro*area)

ggplot(subset(removal,!time %in% c(2116))) + 
  geom_bar(aes(x=time, weight=income/5000000000, fill=assort)) +
  scale_fill_viridis_d(end = 0.9) +
  labs(y=NULL,title = "Income, bil.€/year", x="5-year intervals", fill=NULL ) +
  theme(legend.position = "bottom", axis.text.x = element_text(angle = -90))
```





# Scenario 2
In this example the tree species changes after final felling. Therefore we redefine final felling activity taking into account in addition to volume and age also the dominant species. The change probability depends on region and dominant species.
Volume and age act as before. They move to the smallest classes (vol1=1 and age1=1).
```{r}
transprobs_ff_age_species <- statespace %>%
  select(vol0=vol, age0=age, sp0=sp, region) %>%
  unique %>%
  group_by(region, sp0, vol0, age0) %>%
  reframe(data.frame(vol1=1, age1=1, sp1=c('other','spruce'),
                       prob=case_when(
                         sp0=='other' && region=='South' ~ c(1, 0),
                         sp0=='other' && region=='Middle' ~ c(1, 0),
                         sp0=='other' && region=='North' ~ c(0.8, 0.2), #in other dominated forest
                                        # in northern boreal vegetation zone 20% of final felled area
                                        # change to spruce dominated forest
                         sp0=='spruce' && region=='South' ~ c(0.3, 0.7),
                         sp0=='spruce' && region=='Middle' ~ c(0.2, 0.8),
                         sp0=='spruce' && region=='North' ~ c(0, 1))))
ff_age_species <- define_activity("ff", c("vol", "age", "sp"), transprobs_ff_age_species)
```

The list of activities includes the same noman and thin as in Scenario 1, and the above defined final felling. The same initial state as in Scenario 1 is run with the new list of activities and stored in variable states2.
```{r}
activities2 <- list(noman, thin, ff_age_species)
states2 <- runEFDM(state0, actprob, activities2, 20)
```


The proportion of spruce dominated forest area is estimated for time steps 0, 10 and 20 (years 2016, 2066 and 2116). And the result is presented by the vegetation zones as a map.

```{r}
# Compute proportion of spruces in each time and region
prop_spruces <- states2 %>%
  group_by(region, time) %>%
  summarise(.groups="drop", proportion = 100*sum((sp=="spruce")*area)/sum(area)) %>%
  filter(time %in% c(0, 10, 20)) %>%
  mutate(time = 2016 + 5*time)

# Add bio-geographical regions (MetsaKasvVyoh provided in efdm package) to the data
prop_spruces <- merge(MetsaKasvVyoh, prop_spruces)
prop_spruces %>% ggplot() +
  geom_sf(aes(fill=proportion)) +
  facet_wrap(vars(time)) +
  scale_fill_gradient(low = "white", high = "forestgreen") +
  labs(fill="Proportion" ,title="Proportion of spruce dominated forests, %") +
  guides(fill = guide_colourbar(frame.colour = "grey20",ticks.colour = "grey20")) +
  theme(legend.position = "bottom", axis.text.x = element_text(angle = -90))
```

The map provided by: Finnish Environment Institute

# Scenario 3 - Land use changes

First we add two land use classes to the statespace. Agriculture is stratified according to region and soil type, while other land use is only stratified according to region.
```{r}
statespace3 <- statespace
statespace3$landuse <- "forest"
agriculture <- unique(statespace3 %>% mutate(sp=0, vol=0, age=0, landuse="agriculture"))
other <- unique(statespace3 %>% mutate(soil=0, sp=0, vol=0, age=0, landuse="other"))
statespace3 <- rbind(statespace3, agriculture, other)
```

We use separate activities for deforestation to each land use class.
Variables (vol, age, sp) not used by the agriculture are set to 0.
Soil type and region are not changing as a result of deforestation to agriculture.
```{r}
transprobs_defor_to_agri <- statespace3 %>%
  filter(landuse=="forest") %>%
  select(vol0=vol, age0=age, sp0=sp,landuse0=landuse) %>%
  unique %>%
  mutate(vol1=0, age1=0, sp1=0, landuse1="agriculture", prob=1)
defor_to_agri <- define_activity("defor_to_agri",
                                 c("vol", "age", "sp", "landuse"),
                                 transprobs_defor_to_agri)
```

Deforestation to other land use also changes soil type to 0.
```{r}

transprobs_defor_to_other <- statespace3 %>%
  filter(landuse=="forest") %>%
  select(soil0=soil, vol0=vol, age0=age, sp0=sp,landuse0=landuse) %>%
  unique %>%
  mutate(soil1=0, vol1=0, age1=0, sp1=0, landuse1="other", prob=1)
defor_to_other <- define_activity("defor_to_other",
                                  c("soil", "vol", "age", "sp", "landuse"),
                                  transprobs_defor_to_other)
```

Afforestation only applies to agriculture.
Volume and age classes start from 1 and the area is split evenly to spruce and other species.
```{r}
transprobs_aff <- statespace3 %>%
  filter(landuse=="agriculture") %>%
  select(soil, vol0=vol, age0=age, region, sp0=sp, landuse0=landuse) %>%
  mutate(vol1=1, age1=1, sp1="other", landuse1="forest", prob=1)
transprobs_aff <- rbind(transprobs_aff %>% mutate(sp1="spruce", prob=0.5),
                        transprobs_aff %>% mutate(sp1="other", prob=0.5))
affor <- define_activity("affor",
                         c("vol", "age", "sp", "landuse"),
                         transprobs_aff)
```

A donothing activity is used for non-forest land uses, when there is nothing forestry related going on.
```{r}
donothing <- define_activity("donothing", character())
activities3 <- list(noman, thin, ff, defor_to_other, defor_to_agri, affor, donothing)
```

adding land use information to state 
```{r}
state03 <- state0
state03$landuse <- "forest"
state03 <- rbind(state03,  agriculture %>% mutate(area=rep(c(1552000/2,997000/2,76000/2),each=2)),
                 other %>% mutate(area=c(721000,507000,112000)))
```

Activity probabilities for new land uses 
```{r}
actprob3 <- actprob
actprob3$landuse <- "forest"
# Define probabilities for new activities
actprob3$defor_to_other <- 0.0002
actprob3$defor_to_agri <- 0.00025
actprob3$affor <- 0
actprob3$donothing <- 0
# Normalise probabilities after adding new activities
actnames <- c("noman", "thin", "ff", "defor_to_other", "defor_to_agri", "affor", "donothing")
actprob3[actnames] <- actprob3[actnames]/rowSums(actprob3[actnames])
# Define activity probabilities also for agriculture and other land uses
actprob3 <- rbind(actprob3, 
        agriculture %>% mutate(thin=0, ff=0, noman=0, defor_to_other=0, defor_to_agri=0, affor=0.0005, donothing=0.9995),
        other %>% mutate(thin=0, ff=0, noman=0, defor_to_other=0, defor_to_agri=0, affor=0, donothing=1))
```


```{r}
states3 <- runEFDM(state03, actprob3, activities3, 20)
```

```{r}
#pie charts of land use areas
LUareas <- states3 %>%
  group_by(time, landuse) %>%
  summarise(.groups="drop", area = sum(area)) %>%
  ungroup()
LUareas <- LUareas %>%
  mutate(time = factor(time, labels = seq(2016,2116,5)))
totalarea <- sum(state03$area)
LUareas %>% filter(time %in% c("2016","2066","2116")) %>%
  mutate(area = area/totalarea) %>%
  ggplot() +
  geom_bar(aes(x="",y=area, fill=landuse), stat="identity", width=1, color = "white") +
  scale_fill_viridis_d(end = 0.9) +
  coord_polar("y", start=0) +
  facet_wrap(vars(time)) +
  labs(y=NULL,fill="Land use", x=NULL)+
  theme(axis.text.x = element_blank(),
        axis.ticks = element_blank()) +
  geom_text(aes(x="",y = rep(c(0.03, 0.33, 0.95), 3), label = round(area*100,1)),
            nudge_x=0.25, size=2.25,
            color=rep(c("grey20","white","white"), 3), fontface="bold") +
  theme(legend.position = "bottom")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/activity.r
\name{define_activity}
\alias{define_activity}
\title{Define an activity}
\usage{
define_activity(name, dynamicvariables, transprobs = NULL, probname = name)
}
\arguments{
\item{name}{Name of activity used in reporting}

\item{dynamicvariables}{Names of variables where changes happen}

\item{transprobs}{Transition probabilities (\code{data.frame})}

\item{probname}{(optional) Name of activity in activity probabilities data}
}
\value{
An activity definition
}
\description{
Define an activity
}
\details{
The set of activities in EFDM defines all possible alternatives for a forest
stratum to develop during a scenario run step. Therefore activities are not only
limited to forest treatments and management actions such as thinnings and final fellings
but should also include 'no management' i.e. growth, if applicable. An activity may
also be something else affecting the development, for example, a forest hazard:
snow, wind, drought, pest damage etc.

\code{name} is used as the name of the activity in the result of \code{\link{runEFDM}}
and it is used to identify the activity probability.
If multiple activities should be reported under a common name, \code{actprobname} can
be used to specify the activity name in activity probability data.

\code{dynamicvariables} are those (state space) variables which change as
a result of the activity
Typically an activity affects on the age, volume or stem count of the
forest, but an activity may also, for example, change land-use and then
a variable related to land-use categories is essential.

The \code{transprobs} should be a \code{data.frame} with columns:
\itemize{
\item pairs of dynamic variables. For example, if the dynamic variables are
  \code{vol} and \code{age}, then transprobs should have columns
  \code{age0}, \code{age1}, \code{vol0}, \code{vol1}.
\item the probability of transition \code{prob}
\item and possibly other variables affecting the transition probabilities.
  For example \code{soiltype}, \code{treespecies}, etc.
}
\code{\link{estimatetransprobs}} can be used to estimate transition probabilities
from a similar set of pair data of observations.
}
\examples{
define_activity("nomanagement", c("vol", "age"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-example.r
\docType{data}
\name{example}
\alias{example}
\title{Example dataset.}
\format{
A list of data frames:
\describe{
  \item{actprob}{Activity probabilities}
  \item{noman_pairs}{Pair data for growth without management activities}
  \item{thin_pairs}{Pair data for thinning}
  \item{initial_state}{Initial forest state}
  \item{drain_coef}{Coefficient to transform harvested areas into harvest accumulation by timber assortments}
  \item{vol_coef}{Coefficients to transform volume classes into volumes m3/ha}
  \item{income_coef}{Coefficients to transform harvest accumulation into income}
}
}
\usage{
example
}
\description{
A list containing the necessary datasets for EFDM forest scenario example.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-example.r
\docType{data}
\name{MetsaKasvVyoh}
\alias{MetsaKasvVyoh}
\title{Finnish bio-geographical regions}
\format{
An sf-object:
\describe{
  \item{region}{Factor with three levels: North, Middle, South}
  \item{geometry}{Polygon, each region is composed of multiple polygons}
}
}
\usage{
MetsaKasvVyoh
}
\description{
A low resolution copy of the finnish bio-geographical regions.
The original shapefile was provided by Finnish Environment Institute.
See \url{https://ckan.ymparisto.fi/fi/dataset/metsakasvillisuusvyohykkeet}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior.r
\name{prior_ff}
\alias{prior_ff}
\alias{prior_grow}
\title{Priors for \code{\link{estimatetransprobs}}}
\usage{
prior_ff()

prior_grow(variable, howmuch = 1)
}
\arguments{
\item{variable}{Name of the variable to grow}

\item{howmuch}{Amount of growth}
}
\value{
Return value is used by \code{\link{estimatetransprobs}} to provide
prior information on the transition probabilities.
}
\description{
Priors for \code{\link{estimatetransprobs}}
}
\details{
\code{prior_ff} moves the forest area to the smallest classes of the given
dynamic variables of the forest stratum.

\code{prior_grow} moves the forest to another class given by increasing
\code{variable} by \code{howmuch}.
}
\examples{
statespace <- expand.grid(a=1:2, b=1:2, vol=1:15, age=1:15)
act <- define_activity("test", c("vol", "age"))
act1 <- estimatetransprobs(c("vol", "age"), NULL, statespace, by=c("a", "b"),
  prior=prior_ff())
act2 <- estimatetransprobs(c("vol", "age"), NULL, statespace, by=c("a", "b"),
  prior=prior_grow("age"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estim.r
\name{estimatetransprobs}
\alias{estimatetransprobs}
\title{Estimate Transition Probabilities from Pairdata}
\usage{
estimatetransprobs(
  dynamicvariables,
  pairdata,
  statespace,
  factors = character(),
  by = character(),
  prior = "nochange"
)
}
\arguments{
\item{dynamicvariables}{The names of the dynamic variables, \code{character} vector}

\item{pairdata}{\code{data.frame} Observed transitions}

\item{statespace}{\code{data.frame} or a \code{list} of two \code{data.frame}s if the state space changes as a result of the activity}

\item{factors}{\code{character} Variables used by the activity}

\item{by}{\code{character} Variables that split the state space}

\item{prior}{function or character}
}
\value{
Transition probabilities as a \code{data.frame}
}
\description{
Estimate Transition Probabilities from Pairdata
}
\details{
Transition probabilities 'move' the forest areas allocated in the cells of state matrix
from the initial states in the beginning of a EFDM run step to the end position. This end
position will be the initial state of the next EFDM step. Length of a step (=time)
in the EFDM run is typically determined by the pairdata. It is the time difference of
tree observations. Note that the pairdata can be also constructed from single observation,
if the other (pair) observation is estimated or modelled.

Each activity needs to have a transition probability. If no pairdata is available,
transition probability matrices can be based entirely on a prior defined with
expert knowledge.

The estimation uses an iterative Bayesian algorithm that is explained in
\url{https://github.com/ec-jrc/efdm/blob/master/documents/EFDMinstructions/Seija_Mathematics_behind_EFDM.pdf}.
The transition probability estimate is the proportion of observed transitions
divided by the number of all transitions from the same starting state.
\code{prior} gives the number of prior transitions. For each \code{factor}
variable the transitions are counted in classes of all factors before the
current factor. The "most important" observations (having all classes right)
is counted \code{length(factors)} times, the second most important observations
are counted \code{length(factors)-1} times and so on.

If pairdata is NULL prior is used by itself.

Observations should have 'factor' and 'by' variables and statepairs with 0 and 1
suffixes to indicate before and after observations.

The estimation algorithm uses information across 'factor' variables, but not
across 'by' variables.

\code{prior} can either \code{character} or \code{function}.
\itemize{
\item "nochange" implies that there is one observation where state doesn't change
\item "uninformative" when no observations are given all states are as likely
\item \code{function(A, dynvar1, dynvar0)} where A is an array of zeros with
  dimnames(A) <- c(dynvar1, dynvar0). The function should fill A with the number of
  prior transitions and return it.
}

The statespace is used to fill in the transitions where there are no observations.
In special cases the statespace may change as a result of the activity.
For example changing the tree species might lead to change in the volume classification used.

This function assumes that the dynamic variables are coded as integers starting with 1.
Other variables are not restricted.
}
\examples{

# Estimation can use observed transitions with different levels of factors.
statespace <- expand.grid(a=1:2, b=1:2, vol=1:5)
pairdata <- data.frame(a=c(1,1,2,2), b=c(1,2,1,2), vol0=c(1,1,1,1), vol1=c(2,3,4,5))
state0 <- statespace
actprob <- statespace
actprob$test <- 1
state0$area <- 0
state0$area[1] <- 1

# With by=c("a", "b") there are two observations: one from prior and the other
# from the exact combination of class levels.
probs <- estimatetransprobs("vol", pairdata, statespace, by=c("a", "b"), prior="nochange")
act1 <- define_activity("test", c("vol"), probs)
runEFDM(state0, actprob, list(act1), 1)

probs <- estimatetransprobs("vol", pairdata, statespace, factors="a", by="b", prior="nochange")
act2 <- define_activity("test", c("vol"), probs)
runEFDM(state0, actprob, list(act2), 1)

probs <- estimatetransprobs("vol", pairdata, statespace, factors="b", by="a", prior="nochange")
act3 <- define_activity("test", c("vol"), probs)
runEFDM(state0, actprob, list(act3), 1)

# The order of variables in factors argument specifies the order of importance.
# Observation that differ in the first variable are counted more times.
probs <- estimatetransprobs("vol", pairdata, statespace, factors=c("a", "b"), prior="nochange")
act4 <- define_activity("test", c("vol"), probs)
runEFDM(state0, actprob, list(act4), 1)

probs <- estimatetransprobs("vol", pairdata, statespace, factors=c("b", "a"), prior="nochange")
act5 <- define_activity("test", c("vol"), probs)
runEFDM(state0, actprob, list(act5), 1)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.r
\name{runEFDM}
\alias{runEFDM}
\title{Run European Forestry Dynamics Model}
\usage{
runEFDM(state0, actprob, activities, n, check = TRUE, format = 1)
}
\arguments{
\item{state0}{\code{data.frame} Initial state}

\item{actprob}{\code{data.frame} Activity probabilities}

\item{activities}{\code{list} A list of activities}

\item{n}{\code{integer} Number of time steps required}

\item{check}{Check input arguments for consistency.}

\item{format}{??}
}
\value{
\code{data.frame} State of each time step divided by activities
}
\description{
Run European Forestry Dynamics Model
}
\details{
This is the actual scenario running function, which projects the initial
forest state n time steps to the future.

Activities are defined using \code{\link{define_activity}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/activity.r
\name{transprobs<-}
\alias{transprobs<-}
\alias{transprobs}
\title{Transition probabilities of an activity}
\usage{
transprobs(act) <- value

transprobs(act)
}
\arguments{
\item{act}{Activity definition}

\item{value}{\code{data.frame} of transition probabilities}
}
\value{
\code{data.frame} where prob is the transition probability from current
state (with suffix 0) to next state (with suffix 1).
}
\description{
Functions to get or set the transition probabilities of an activity
}
\details{
The \code{value} should be a \code{data.frame} defining the transition probabilities
between dynamic variables of the activity. See \code{\link{define_activity}}
for details
}
\examples{
act1 <- define_activity("test", c("vol"))
transprobs(act1) <- data.frame(vol0 = 1:5, vol1=c(2:5, 5), prob=1)
transprobs(act1)

if(require("ggplot2")) {
  ggplot(transprobs(act1)) + geom_raster(aes(x=vol0, y=vol1, fill=prob))
}
}
